pro adjustzpt, catalog, reference, outcat=outcat, rename=rename, refunc=refunc, nir=nir, filter=filter, adjusted=adjusted
 
  ; Re-set the zeropoint of catalog based on input number (if int/float) or by
  ;   comparison to magnitudes in reference (if specified, otherwise SDSS/Landolt)
  ; Overwrites catalog by default, else prints to outcat (rename deletes input file on success)

  if n_elements(reference) eq 0 then zpt = getzptoffset(catalog, rms=rms, unc=unc, /zpt, getref=catname, nir=nir, filter=filter)  ; this works correctly.
  if n_elements(reference) eq 1 then begin
        if size(reference,/tn) eq 'STRING' then begin
            zpt = getzptoffset(catalog, reference, rms=rms, unc=unc, /zpt, getref=catname, nir=nir, filter=filter)                ; this does NOT.
        endif else begin
            zpt = reference
            catname = 'zeropoint'
        endelse
  endif
 
  adjusted = 0
  if finite(zpt) eq 0 then return  ; unsuccessful [no catalog to compare to, usually]
  if n_elements(refunc) eq 0 then refunc = 0

  cat = readcat(catalog, h)
  oldzeropt = sxpar(h,'ZEROPT', comment=zptcmt)
  sxaddpar, h, 'OLDZPT', oldzeropt, 'Zeropoint used before calibration', before='ZEROPT'
  dzpt = zpt - oldzeropt
  if n_elements(zptcmt) gt 0 then sxaddpar, h, 'ZEROPT', oldzeropt + dzpt, zptcmt $ 
                             else sxaddpar, h, 'ZEROPT', oldzeropt + dzpt  ; this shouldn't be necessary but sxaddpar is buggy
  if n_elements(rms) gt 0 then begin
    sxaddpar, h, 'ZPTRMS', rms, 'RMS of match to catalog stars'
    sxaddpar, h, 'ZPTUNC', sqrt(unc^2 + sxpar(h,'APUNC')^2 + refunc^2), 'Uncertainty of zeropoint'
  endif
  cat.mag += dzpt
  if n_elements(catname) gt 0 then sxaddpar, h, 'PHOTCAL', removepath(catname), 'Catalog used to determine zeropoint'
  if n_elements(filter) gt 0 then calfilt = filter[0] else calfilt = sxpar(h,'FILTER')
  sxaddpar, h, 'CALFILT', calfilt, 'Catalog filter used to compute zeropt'
  if n_elements(outcat) eq 0 then outcatfile = catalog else outcatfile = outcat

  writecat, outcatfile, cat, h
  if keyword_set(rename) and file_test(outcatfile) and (catalog ne outcatfile) then spawn, 'rm -f '+catalog
  adjusted = 1

end

