pro combinesum, filelist, outfile, masklist=masklist, verbose=verbose, expkey=expkey, yshift=yshift, crclean=crclean, noweight=noweight, silent=silent

; Additive combination, mainly for sky-subtracted 2D spectra.  Does some outlier rejection.

if n_elements(outfile) eq 0 then begin
  filearr = strsplit(filelist[0],'.', /extract)
  fileroot = filearr[0]
  outfile = fileroot + 'sum.fits'
endif

if n_elements(filelist) eq 1 and strpos(filelist[0],'.fit') eq -1 then begin
  files = grabcolumn(filelist,0,/str)
  files = files[where(files ne '')]
endif else begin
  files = filelist
endelse

nfiles = n_elements(files)
nmid = nfiles/2

if n_elements(expkey) eq 0 then expkey = 'EXPTIME'

refdata = $;readfits(files[nmid], h, /silent)
          mrdfits(files[nmid], 0, h, /silent)
s = size(refdata)
nx = s[1]
ny = s[2]

data = fltarr(nx, ny, nfiles)
skydata = fltarr(nx, ny, nfiles) ; with sky
maskdata = bytarr(nx, ny, nfiles)
remaskdata = bytarr(nx, ny, nfiles)
exptime = fltarr(nfiles)
airmass = fltarr(nfiles)
inmeds = fltarr(nfiles)
ut = strarr(nfiles)

fits_info, files[0], /silent, n_ext=nextensions

for e = 0, nextensions do begin

  for f = 0, nfiles-1 do begin
    if keyword_set(verbose) then print, files[f]
    indata = $ ;readfits(files[f], h, /silent)
            mrdfits(files[f], e, h, /silent)
    if n_elements(masklist) ge nfiles then inmask = readfits(masklist[f], hmask, /silent)
    maskname = clip(sxpar(h,'BPM'))
    if e eq 0 and maskname ne '0' and file_test(maskname) then inmask = mrdfits(maskname,0, hmask, /silent)
    if e eq 0 and nextensions ge 2 and keyword_set(crclean) then insky = mrdfits(files[f],1, hmask, /silent)

    if n_elements(yshift) gt 0 then begin
      ysh = round(yshift[f])
      if ysh ge ny then begin
        print, '  ERROR: yshift > ny; no overlap between frames.'
        return
      endif
      if ysh ne 0 then begin
        ;print, files[f], ysh
        indata = shift(indata,0,ysh)
        if ysh gt 0 then indata[*,0:ysh-1]     = !values.f_nan   ; these appear to "stick" into the output if other frame pixel is bad...
        if ysh lt 0 then indata[*,ny+ysh:ny-1] = !values.f_nan
      endif
    endif else begin
      if keyword_set(silent) eq 0 then print, 'No spatial shift applied.'
      yshift = 0+fltarr(nfiles)
    endelse
    if f eq 0 then h0 = h
    data[*,*,f] = float(indata)
    if e eq 0 and n_elements(inmask) gt 0 then maskdata[*,*,f] = inmask
    if e eq 0 and n_elements(insky) gt 0 then skydata[*,*,f] = insky + indata
    airmass[f] = sxpar(h,'AIRMASS')
    exptime[f] = sxpar(h,expkey)
    ut[f] = sxpar(h,'UT')
  endfor

  relexptime = exptime/median(exptime)
  comb = fltarr(nx, ny)

  if e eq 0 and keyword_set(silent) eq 0 then print, '  Sum-combining...'
  i = 0l
  for y = 0, ny-1 do begin    
    for x = 0, nx-1 do begin  ; x is the fast array and should be iterated inside
      i = i + 1 ; needed only for the verbose option
      if keyword_set(crclean) and e eq 0 and nextensions gt 1 and e eq 0 then begin     ; does a little bit of CR filtering on its own
         ww = where(maskdata[x,y,*] eq 0 and finite(data[x,y,*]), ctww) ; nearly everything!
         if ctww gt 0 then begin
            low = (-median(-data[x,y,ww]/relexptime[ww])) > 40  ; equivalent to min(data) for n=2
            dev = sqrt(max(skydata[x,y,ww]/min(relexptime[ww])) > 40)
            cr  = where(data[x,y,*] gt (low+8*dev)*relexptime+20 and maskdata[x,y,*] eq 0, ct)
            if ct gt 0 then begin
                 if keyword_set(verbose) then begin
                 print, '-----'
                 print, 'low = ', low
                 print, 'dev = ', dev
                 print, (data[x,y,*]) [*]
                 print, (data[x,y,*]/relexptime) [*]
                 print, (skydata[x,y,*]) [*]
                 print, ' Flagged ', clip(x)+','+clip(y)+','+clip(cr[0]), ' = ', skydata[x,y,cr[0]]
               endif
               remaskdata[x, y, cr] = 1
               if y lt ny-1 then remaskdata[x, y+1, cr] = 1  ; also flag neighbors
               if y gt 0    then remaskdata[x, y-1, cr] = 1
               if x lt nx-1 then remaskdata[x+1, y, cr] = 1
               if x gt 0    then remaskdata[x-1, y, cr] = 1
            endif
         endif
      endif
      comb[x, y] = total(data[x, y, *]) ; mediansigclip?
      if i mod 157557 eq 0 then begin
         st = stddev(data[x, y, *])
         val = comb[x, y]
         if keyword_set(verbose) then print, clip(x,5), clip(y,5), 'stdev=',clip(st,6), '  val=',clip(val,6);, '  stdev/sqrt(val)=',fpr(st/sqrt(val),3.2) 
      endif
    endfor
  endfor
  
  maskdata = maskdata > remaskdata
  
  addmask = total(maskdata, 3)
  
  w = where(addmask gt 0 or finite(comb) eq 0)
  for i = 0L, n_elements(w)-1 do begin
    if w[i] eq -1 then continue
    val = 0.
    itime = 0.
    x = w[i] mod nx
    y = w[i] / nx
    for f = 0, nfiles-1 do begin
       if finite(data[x, y, f]) and maskdata[x,y,f] eq 0 then begin
          val += data[x, y, f]
          itime += exptime[f]
       endif
    endfor
    if itime gt 0 and itime lt total(exptime) then begin
       val *= total(exptime)/itime  ; scale
       comb[x,y] = val
    endif
  endfor
  
  if e eq 0 then begin
    for f = 0, nfiles-1 do begin
      sxaddpar, h0, 'SFILE'+strtrim(string(f+1),2), files[f], f eq 0 ? 'summed 2D frame file name' : ''
    endfor
    for f = 0, nfiles-1 do begin
      sxaddpar, h0, 'YSHIFT'+strtrim(string(f+1),2), yshift[f]
    endfor
    sxaddpar, h0, 'BZERO', 0
    sxaddpar, h0, 'BSCALE', 1
    sxaddpar, h0, 'NSUM', nfiles
    sxaddpar, h0, 'EXPTIME',  total(exptime)
    sxaddpar, h0, 'TOTALEXP', total(exptime)
    sxaddpar, h0, 'AIRMASS', mean(airmass)
    sxaddpar, h0, 'DATE_END', sxpar(h, 'DATE_END')
    for f = 0, nfiles-1 do sxaddpar, h0, 'EXPT'+strtrim(string(f+1),2), exptime[f]
    for f = 0, nfiles-1 do sxaddpar, h0, 'AIRMASS'+strtrim(string(f+1),2), airmass[f]
    for f = 0, nfiles-1 do sxaddpar, h0, 'UT'+strtrim(string(f+1),2), ut[f]
  endif
  
  get_date, now
  hist = 'Processed by combinesum '+now
  sxaddhist, hist, h0

  if e eq 0 then mwrfits, comb, outfile, h0, /create, /silent $
            else mwrfits, comb, outfile, h0, /silent
  ;          else writefits, outfile, comb

endfor

if keyword_set(noweight) eq 0 then begin
  outweightfile = repstr(outfile,'.fits','.weight.fits')
  if outweightfile eq outfile then outweightfile += '.weight'
  writefits, outweightfile, addmask, hmask
endif

if keyword_set(silent) eq 0 then print, '  Written to ', outfile

end

