pro mergecat, files, outfile=outfile, magcol=magcol, filters=filters

   ; Merge several catalogs into a catalog table (not into a catalog structure, sorry)
   ; and print to disk.

   ; magcol and filters are more or less hacks to make this work with other catalog formats

   if n_elements(outfile) gt 0 then dofile = 1 else dofile = 0

   ncat = n_elements(files)

   for f = 0, ncat-1 do begin  
      if file_test(files[f]) eq 0 then begin
         print, files[f], ' not found.'
         continue
      endif
      newcat = readcat(files[f],h, addunc=0.001, magcol=magcol)
      nnew = n_elements(newcat)
      if n_elements(filters) gt 0 then filt = filters[f] $
                                  else filt = sxpar(h,'FILTER', count=ct)
      newfilt = replicate(filt, nnew)
      if f eq 0 then mastercat = newcat  else  mastercat = [mastercat, newcat]
      if f eq 0 then mcatfilt  = newfilt else  mcatfilt  = [mcatfilt, newfilt]
   endfor

   nc = n_elements(mastercat)
   starid = lonarr(nc) - 1

   cosdec = cos(median(mastercat.dec)*!pi/180.)
   s = 0
   for a = 0, nc-1 do begin
     if starid[a] eq -1 then begin
        starid[a] = s  ; is the star unlabeled?  label it.
        s = s + 1
     endif

     neighbors = findneighbors(mastercat.ra*cosdec, mastercat.dec, a, mindist=1./3600., count=ct)
      ; assuming that anything with 1" of the actual star is filtered out already, but probably shouldn't assume this?
     if ct gt 0 then starid[neighbors] = starid[a]
   endfor

   ns = s
   filters = unique(mcatfilt)
   filters = filters[sort(filtwv(filters))]
   nf = n_elements(filters)
   
   nan = !values.f_nan
   mergedcat = replicate({ra:0.0d, dec:0.0d, $
                         mag:replicate(nan,nf), unc:replicate(nan,nf), $
                         rms:replicate(nan,nf), ndet:replicate(0,nf)}, ns) ; mimics readcalibration

   for s = 0, ns-1 do begin
      ws = where(starid eq s, ct)
      mergedcat[s].ra = median([mastercat[ws].ra], /even)
      mergedcat[s].dec = median([mastercat[ws].dec], /even)
      for f = 0, nf-1 do begin
         wsf = where(starid eq s and mcatfilt eq filters[f], ct)
         mergedcat[s].ndet[f] = ct
         if ct eq 0 then continue
         mergedcat[s].mag[f] = median([mastercat[wsf].mag], /even)
         mergedcat[s].unc[f] = wmeanunc([mastercat[wsf].unc])
         if ct gt 1 then begin
            mergedcat[s].rms[f] = n_elements(mastercat[wsf].mag) gt 1 ? stdev(mastercat[wsf].mag) : nan
            chi2dof = total( ((mergedcat[wsf].mag-mastercat[wsf].mag)/mergedcat[wsf].unc)^2) / (ct-1.)
            mergedcat[s].unc[f] *= (chi2dof > 1.4) ; 1.4 is a mean->median fudge factor
         endif
      endfor
   endfor

   medmag = fltarr(ns)
   for s = 0, ns-1 do begin
      medmag[s] = median([mergedcat[s].mag[*]])
   endfor
   mergedcat = mergedcat[sort(medmag)]

   if dofile then openw, 1, outfile
   header = '#ra         dec        '
   fclip = clip(filters,2)
   for f = 0, nf-1 do header += ' '+clip(fclip[f],6)+' '+clip('e_'+fclip[f],5)+' '+clip('n_'+fclip[f],4)
   if dofile then printf, 1, header else print, header
   for s = 0, ns-1 do begin
      line = fpr(mergedcat[s].ra,3.7) + ' ' + fpr(mergedcat[s].dec,3.7)
      for f = 0, nf-1 do begin
         magstr = finite(mergedcat[s].mag[f]) ? fpr(mergedcat[s].mag[f],2.3) : ' -    '
         uncstr = finite(mergedcat[s].mag[f]) ? fpr(mergedcat[s].unc[f],1.3) : '-    '
         nstr   = finite(mergedcat[s].mag[f]) ? clip(string(mergedcat[s].ndet[f],'(I3)'),4) : '-   '
         line += ' ' + magstr + ' ' + uncstr + ' ' + nstr
      endfor
      if dofile then printf, 1, line else print, line
   endfor


   close, 1

end

