pro mkfringe, filenames, outfile=outfile, scale=scale, keyscale=keyscale, quick=quick, removeobjects=removeobjects, objthresh=objthresh, objbuffer=objbuffer, $
 halothresh=halothresh, halobuffer=halobuffer


  ; scale specifies the expected fringe amplitude explicitly - else use median sky level

  ; Make a fringe frame

  if n_elements(objthresh) eq 0 then objthresh = 6
  if n_elements(objbuffer) eq 0 then objbuffer = 12

  if n_elements(filenames) eq 1 then begin
    filename = filenames

    if strpos(filename,'.fit') eq -1 then begin
      if strpos(filename, '.txt') eq -1 and $
         strpos(filename,'.lis') eq -1 then begin
        print, 'File must be fits (.fits) or a list (.txt, .lis)'
        return
      endif
      filenames = strarr(200)
      i = 0
      openr, 1, filename
      iline = ''
      while not eof(1) do begin
        readf, 1, iline
        if strmid(iline,0,1) ne '#' then  filenames[i] = iline
        i = i + 1
      endwhile
      close, 1
      filenames = filenames[0:i-1]
    endif
  endif

   nim = n_elements(filenames)
   im = mrdfits(filenames[0], 0, h, /silent, /fscale)
   nx = (size(im)) [1]
   ny = (size(im)) [2]
   imstack = fltarr(nx,ny,nim)

   fringeim = im*0

   ; older sigma clipping system
   
   for f = 0, nim-1 do begin
        if file_test(filenames[f]) eq 0 then begin
           print, 'File ' , filenames[f], ' not found.' 
           return
        endif
        if f gt 0 then im = mrdfits(filenames[f], 0, h, /silent, /fscale)

        if keyword_set(removeobjects) then maskobjects2, im, /nan, objthresh=7., halothresh=halothresh, halobuffer=halobuffer
        ;writefits, 'x'+removepath(filenames[f]), im, h

        immed = median(im, /even)
        w = where(finite(im))
        imthresh = immed + 5*stdev(im[w])
        imclipmed = median(im[where(im lt imthresh,ctclip)], /even)

        if n_elements(keyscale) gt 0 then begin
          imscale=float(sxpar(h,keyscale))
        endif else begin
          if n_elements(scale) gt 0 then begin
             imscale=scale[f] 
          endif else begin
             imscale=imclipmed
          endelse
        endelse

        print, removepath(filenames[f]), imscale
        ;writefits, 'f'+clip(f)+'.fits', (im-imclipmed)/imscale

        imstack[*,*,f] = (im-imclipmed)/imscale
     ;print, im[f], immed, imclipmed, nx*ny-ctclip
   endfor

   nim = intarr(nx, ny)

   if keyword_set(quick) then begin
     fringeim = median(imstack, dimension=3, /even)
   endif else begin
     for x = 0, nx-1 do begin
       for y = 0, ny-1 do begin
          pixstack = imstack[x,y,*]
          wf = where(finite(pixstack), ct)
          nim[x,y] = ct
          if ct le 1 then begin
            if ct eq 0 then fringeim[x,y] = !values.f_nan
            if ct eq 1 then fringeim[x,y] = pixstack[wf]
          endif else begin
            pixmed = median(pixstack[wf], /even)
            pixstdev = stdev(pixstack[wf])
            pixclipmed = median(pixstack[where(pixstack le pixmed + 3*pixstdev)], /even)
            fringeim[x,y] = pixclipmed
          endelse
       endfor
     endfor

   endelse

   for f = 0, n_elements(filenames)-1 do begin
      sxaddpar, h, 'FRINGE'+strtrim(string(f),2), filenames[f]
   endfor


   if n_elements(outfile) eq 0 then outfile = 'fringe.fits'
   writefits, outfile, fringeim, h
   mwrfits, nim, outfile

end

