function readspec, filename, h, header=header, timenorm=timenorm, cutoff=cutoff

  ; reads an ASCII spectrum in Perley's standard format, analogous to a fits file.

  close, 1
  if file_test(filename) eq 0 then begin
     print, 'File '+filename+' not found!'
     return, ['']
  endif
  if strpos(filename,'.fit') ge 0 then begin
     print, 'Cannot process fits files.'
     return, ['']
  endif

  nl = countlines(filename)  
  spec = replicate({wav:0.0d, flux:0.0, sky:0.0, unc:0.0, pix:0L, ypix:0., resp:0.0},nl)
  header = ['']
  fluxcol = -1 & wavcol = -1 & skycol = -1 & unccol = -1 & xcol = -1 & ycol = -1 & respcol = -1
  
  openr, 1, filename
  inline = ''
  i = 0
  for l = 0l, nl-1 do begin
     readf, 1, inline
     inline = clip(inline)
     if strlen(inline) eq 0 then continue
     if strmid(inline,0,2) eq '##' and strpos(inline,'flux') gt 3 then begin
        ; column IDs
        cols = (strsplit(inline,/extract)) [1:*]
        wavcol = (where(strpos(cols,'wave') ge 0)) [0]
        fluxcol = (where(cols eq 'flux')) [0]
        skycol = (where(strpos(cols,'sky') ge 0)) [0]
        unccol = (where(strpos(cols,'unc') ge 0)) [0]
        xcol = (where(strpos(cols, 'xpix') ge 0)) [0]
        ycol = (where(strpos(cols, 'ypix') ge 0)) [0]
        respcol = (where(strpos(cols,'resp') ge 0)) [0]
        ; print, cols
        ; print, wavcol, fluxcol, skycol, unccol, xcol, ycol, respcol
     endif
     if strmid(inline,0,1) eq '#' then begin
        inline = clip(strmid(inline,1))
        hline = processheaderline(inline)
        if hline ne '' then begin
          header = [header, clip(hline,78)]  ; might not be strictly valid
        endif
     endif else begin
        splitline = strsplit(inline, /extract)
        ni = n_elements(splitline)
        if ni eq 0 then continue

        if fluxcol eq -1 then begin
          ; no column header: guess based on number of columns
          if ni eq 1 then begin
             fluxcol = 0
          endif else begin
            wavcol = 0
            fluxcol = 1
            if ni ge 3 then skycol = 2
            if ni ge 4 then unccol = 3
            if ni ge 5 then xcol = 4
            if ni ge 6 then ycol = 5
            if ni ge 7 then respcol = 6
          endelse
        endif

        if wavcol ge 0  and wavcol lt ni  then spec[i].wav  = float(splitline[wavcol])
        if fluxcol ge 0 and fluxcol lt ni then spec[i].flux = float(splitline[fluxcol])
        if skycol ge 0  and skycol lt ni  then spec[i].sky  = float(splitline[skycol])
        if unccol ge 0  and unccol lt ni  then spec[i].unc  = float(splitline[unccol])
        if xcol ge 0    and xcol lt ni    then spec[i].pix  = float(splitline[xcol])
        if ycol ge 0    and ycol lt ni    then spec[i].ypix = float(splitline[ycol])
        if respcol ge 0 and respcol lt ni then spec[i].resp = float(splitline[respcol])

        if fluxcol eq 0 and xcol eq -1 then spec[i].pix = i

        i += 1
     endelse
  endfor

  spec = spec[0:i-1]

  if keyword_set(timenorm) then begin
       tn = sxpar(header,'EXPTIME', count=ct)
       if ct eq 0 then tn = sxpar(header,'ELAPTIME', count=ct)
       if ct eq 0 then tn = sxpar(header,'EXPOSURE', count=ct)
       if ct eq 0 then begin
         print, 'WARNING:  Cannot normalize exposure time!'
         tn = 1.0
       endif
       spec.flux /= tn
       spec.unc /= tn
       spec.sky /= tn
  endif

  dwav = spec.wav - shift(spec.wav,+1)
  dwav[0] = dwav[1]
  negdwav = where(dwav lt 0, ctnegdwav)
  if ctnegdwav gt 0 and ctnegdwav lt n_elements(dwav) then begin
      print, 'Warning: Wavelength solution for this spectrum is double-valued.'
      print, '     Removing points below the inflection. '
   ; This is a temporary solution; you won't always want this behavior.  Work on the wavelength solver to
   ;   recognize and stop such bad solutions earlier in the process.
      spec = spec[where(dwav gt 0.)]
  endif

  if n_elements(cutoff) ge 1 then begin
     spec = spec[where(spec.wav gt cutoff[0])]
  endif

  close, 1
  h = header
  return, spec

end

