pro realignimage, imname, imrefname, mindist=mindist, redistort=redistort
   ; modified quickalign
   if n_elements(sexcommand) eq 0 then sexcommand = 'sex'
   if n_elements(mindist) eq 0 then mindist = 1.5

   if file_test('temp.param') eq 0 then begin
       params = ['X_IMAGE','Y_IMAGE','ALPHA_J2000','DELTA_J2000','MAG_AUTO',$
                 'MAGERR_AUTO','ELLIPTICITY','FWHM_IMAGE','FLAGS']

       openw, 1, 'temp.param'
       for p = 0, n_Elements(params)-1 do printf, 1, params[p]
       close, 1
   endif

   images = [imrefname, imname]
   for i = 0, 1 do begin
       ; reference can be a catalog file (first two columns are RA/dec)
       if i eq 0 and strpos(images[i],'.fit') eq -1 then begin
         readcol, imrefname, refa, refd, format='f,f', comment='#', /silent
         continue
       endif
       h = headfits(images[i])

       extast, h, hast
       sat = sxpar(h,'SATURATE')

       ;sex, 'images[i] -x '+clip(sat)
       sexcat = repstr(images[i],'.fits','.sex')
       if file_test(sexcat) eq 0 then begin
          sexcmd = getsexpath()
          spawn, sexcmd+' '+images[i]+' -c sex.config -CATALOG_NAME '+sexcat+ ' -SATUR_LEVEL '+clip(sat)
          ;sex, images[i]+' -c sex.config -CATALOG_NAME '+sexcat+ ' -SATUR_LEVEL '+clip(sat)
       endif

       readcol, sexcat, x, y, format='f,f', comment='#', /silent
       xy2ad, x, y, hast, a, d

       if i eq 0 then begin
          refa = a
          refd = d
       endif
   endfor

   matchid = intarr(n_elements(a)) - 1 ; matchid[j] = k

   ; do the matching
   cosdec = cos(median(d)*!pi/180.)
   for j = 0, n_elements(a)-1 do begin
       bestdist = mindist
       for k = 0, n_elements(refa)-1 do begin 
           da = (abs(a[j]-refa[k])*cosdec)*3600.
           if da gt mindist then continue
           dd = (abs(d[j]-refd[k]))*3600.
           if dd gt mindist then continue
           dist = sqrt(da^2 + dd^2)
           if dist lt bestdist then begin
              matchid[j] = k
              bestdist = dist
              ;print, j, '=', k, ' : ', dist
           endif
       endfor
   endfor

 ;  window, 0
 ;  plot, [0], [0], xrange=[min([a, refa]), max([a,refa])], yrange=[min([d, refd]), max([d,refd])], /xsty, /ysty
 ;  oplot, refa, refd, psym=1, color=rgb(255,0,0)
 ;  oplot, a, d, psym=7, color=rgb(0,255,0)

   ;stop

   fixsip = keyword_set(redistort) eq 0
   matched = where(matchid ge 0, ct)
   if ct lt 5 then begin
      print, 'Not enough matches ('+clip(ct)+')'
      return
   endif
   if fixsip eq 0 and ct lt 20 then begin
      print, 'Not enough matches for a full distortion solution ('+clip(ct)+'), solving rotation/offset only'
      fixsip = 1
   endif
   fx = x[matched] ; fitter inputs
   fy = y[matched]
   fra = refa[matchid[matched]]
   fdec = refd[matchid[matched]]

   hnew = fitalign(fx, fy, fra, fdec, h, /crotonly, fixsip=fixsip)
   sxaddpar, hnew, 'ALIGNREF', imrefname, ' Image aligned against'
    
   im = readfits(imname, /silent)
   writefits, fileappend(imname,'e'), im, hnew

end
