function pcol, r, g, b, colors=colors, inv=inv
   if n_elements(colors) eq 0 then return, rgb(r,g,b)   
   if keyword_set(inv) then begin
     r = 255-r
     r = 255-g
     r = 255-b
   endif
   coldist = abs(r-colors[*,0])+abs(g-colors[*,1])+abs(b-colors[*,2])
   w = (where(coldist eq min(coldist))) [0]
   return, w
end

pro trace2d, filename, blocksize=blocksize, xstart=xstart, xrange=xrange, ploty=ploty, tracefitorder=tracefitorder, plotfile=plotfile, tracefile=tracefile, profilefile=profilefile, settracefile=settracefile

   ; new, possibly superior tracing mechanism: uses cross-correlation around the strongest source(s)
   
   ; writes out a function containing trace(x)
   ; also writes out a function containing profile(y) @ xcen

   if file_test(filename) eq 0 then begin
       print, filename, ' not found.'
       return
   endif

   if n_elements(tracefitorder) eq 0 then tracefitorder = 3

   im = readfits(filename, h, /silent)
   nx = (size(im)) [1]
   ny = (size(im)) [2]
   if n_elements(xrange) eq 0 then xrange = [0,nx-1]
   if n_elements(xstart) eq 0 then xstart = mean(xrange)
   if n_elements(blocksize) eq 0 then blocksize = 128
   rx = xrange[1]-xrange[0]

   ; If sky-line file is available, read it in and mask out the sky lines
   skyfile = 1
   if skyfile eq 1 then begin
     origim = im
     sky = mrdfits(filename, 1, /silent)
     med = median(sky[*,ny/2-20:ny/2+20], /even, dimension=2)
     med30 = median(med, 30)
     skycont = fltarr(nx)
     for i = 0, nx-1 do skycont[i] = min(med30[(i-20)>0:(i+20)<(nx-1)] > 0.)  ; should make it not hit telluric bands
     ; window, 0
     ; plot, sky[*,ny/2], yrange=[0,1000]
     ; oplot, skycont, color=rgb(255,0,0)
     for x = 0, nx-1 do sky[x,*] -= skycont[x]
     ; window, 1
     ; plot, sky[*,ny/2]
     smed = median(sky)
     skypix = where(sky gt 64+(smed > 0)*4.5, ct)
     if ct gt 0 then begin
       ; print, '  Removing ', clip(ct), ' bright sky-line pixels'
       im[skypix] = !values.f_nan
     endif
     ; writefits, 'test.fits', im, h
   endif



   ; Make a preliminary measurement of the profile to find where the sources are
   if n_elements(settracefile) eq 0 then begin
    xf0 = xstart-blocksize   ; this assumes trace gradient isn't more than ~1 pixel per 100 pixels
    xf1 = xstart+blocksize
    yy = indgen(ny)
    yv = fltarr(ny)
    for y = 0, ny-1 do yv[y] = median(im[xf0:xf1,y], /even)   
    yv -= median(yv, 15) ; remove residual gradients, should try to clean up skysubtract2 to make this unnecessary
    wf = where(finite(yv), ctfinite)
    if ctfinite lt 4 then for y = 0, ny-1 do yv[y] = median(origim[xf0:xf1,y], /even)   ; sky line removal was overzealous
    removenansnn, yv
    sigclipstats, yv, sigmahi=2.7, sigmalo=3.0, median=yvmed, stdev=yvstdev, mean=yvmean
    yvsmooth = convol(yv, [1,1,1]/3.)
    yvsmoothsig = yvsmooth/yvstdev
    maxsig = max(yvsmoothsig[20:ny-50])
    ; issource = yvsmoothsig gt 2.3 and yvsmoothsig gt 0.45*maxsig and yy gt 20 and yy lt ny-50
    ; sourceypix = where(issource, nysourcepix) 
    sourceypix = (where(yvsmoothsig eq maxsig)) [0]
    ; fit the width?
    ; if nysourcepix le 2 then begin
    if yvsmoothsig[sourceypix] lt 5.0 then begin
       print, '  Could not find any sources within +/-'+clip(blocksize)+' pixels of x=', clip(xstart), '.  Cannot trace.'
       ; plot, yvsmoothsig
       return
    endif
    ;ypeak = where(yvsmooth gt shift(yvsmooth,-1) and yvsmooth ge shift(yvsmooth,1) and $ 
    ;              yvsmooth gt shift(yvsmooth,-2) and yvsmooth ge shift(yvsmooth,2) and issource)
    ;ypeak = ypeak[sort(ypeak)]
    ypeak = sourceypix
 
    ; now create an array that contains only the value of y pixels that might contain sources
    usey = indgen(ny)
    for y = 0, ny-1 do $
       if min(abs(usey[y]-sourceypix)) gt 20 then usey[y] = -1  ; look within 20 pixels of brightest source
    blanky = usey[where(usey lt 0)]
    usey = usey[where(usey ge 0)]
    ; print, usey

    if max(yvsmooth)/yvstdev lt 5.0 then blocksize *= 2
    if max(yvsmooth)/yvstdev gt 100.0 then blocksize /= 2
    if max(yvsmooth)/yvstdev gt 1000.0 then blocksize /= 2
   endif


   nblock = ceil(nx/blocksize)
   startb = round(xstart/blocksize)
   xbc = fltarr(nblock)    ; center position of block
   ; cumshiftarr = fltarr(nblock) + !values.f_nan
   absshiftarr = fltarr(nblock) + !values.f_nan
   maxsigarr   = fltarr(nblock) + !values.f_nan
   x0 = indgen(nblock)*blocksize
   x1 = (indgen(nblock)*blocksize+blocksize-1) < (nx-1)
   xc = (x0 + x1)/2.

   ; Create a median-blocked image
   ; (This is very similar to the "source" image already output by the sky subtractor...)
   blockim = fltarr(nblock, ny)
   for b = 0, nblock-1 do begin
      if xc[b] lt xrange[0] then continue
      yv = fltarr(ny)
      for y = 0, ny-1 do yv[y] = median(im[x0[b]:x1[b],y], /even)   
      wf = where(finite(yv), ctfinite)
      if ctfinite lt 4 then for y = 0, ny-1 do yv[y] = median(origim[x0[b]:x1[b],y], /even)  
          ; sky line removal was overzealous
      removenansnn, yv
      blockim[b,*] = yv
   endfor
   ; evidently this isn't perfect.  Occasionally a column with blockim[b,*] = const leaks through
   ; lsr160112_0043r_PTF_s5.fits is an example


   if n_elements(settracefile) gt 0 then begin
         openr, 3, settracefile
         inline = ''
         readf, 3, inline
         while eof(3) eq 0 and (strlen(inline) eq 0 or strmid(clip(inline),0,1) eq '#') do readf, 3, inline
         close, 3
         tracepar = float(strsplit(inline,/extract))
         tracey = poly(findgen(nx), tracepar)
         usey = 20+indgen(ny-40)
         traceyxc    = poly(xc, tracepar)
   endif


   !p.multi = [0,1,4]
   !p.position = 0
   if n_elements(plotfile) gt 0 then begin
      psopen, plotfile, xsize=7, ysize=7.5, /inch, /encaps
      device, /color
      colors = intarr(256,3)
      colors[1,*] = [255,0,0]
      colors[2,*] = [0,255,0]
      colors[3,*] = [0,0,255]
      colors[4,*] = [255,255,0]
      for c = 5, 50 do begin
          cf = (c-5.)/(50-5.)
          colors[c,*] = fix([255*cf, 255-255*abs(cf-0.5), 255.*(1-cf)])
      endfor
      for c = 51, 255 do begin
          cf = (c-51.)/(255-51.)
          colors[c,*] = fix(255*[cf,cf,cf])
      endfor
      tvlct, colors
   endif else begin
      window, 0, xsize=700, ysize=880
   endelse
   if n_elements(ploty) eq 0 then ploty = [min(usey), max(usey)]

   ; Plot 1: overplotting of all the profiles, no shifting
   plot, [0], [0], xrange=ploty, yrange=[min(blockim[*,10:ny-10]), max(blockim[*,10:ny-10])], /xsty, /ysty, charsize=2
   for b = 0, nblock-1 do begin
      yv = (blockim[b,*]) [*]  ; yv is the (unshifted) profile over this block
      oplot, yv, color=pcol(255*(xc[b]-xrange[0])/rx, 255-255*abs(xc[b]-mean(xrange))/rx, 255.*(xrange[1]-xc[b])/rx, colors=colors)      
   endfor

   if n_elements(settracefile) eq 0 then begin

   ; Plot 2: after shifting
   plot, [0], [0], xrange=ploty, yrange=[0, 1], /xsty, /ysty, charsize=2
   for dir = 0, 1 do begin
      if dir eq 0 then stopb = nblock-1 else stopb = 0
      if dir eq 0 then inc = 1 else inc = -1
      for b = startb, stopb, inc do begin
         if xc[b] lt xrange[0] or xc[b] gt xrange[1] then continue
         yv = (blockim[b,*]) [*]  ; yv is the (unshifted) profile over this block
         yvo = yv
         for y = 0, n_elements(yv)-1 do $ 
            yv[y] -= median([[yvo[((y-7)>0):((y-1)>0)]],yvo[((y+1)<(ny-1)):((y+7)<(ny-1))]],/even) ; remove gradients
         sigclipstats, yv, sigmahi=2.7, sigmalo=3.0, median=yvmed, stdev=yvstdev, mean=yvmean
         yvsmooth = convol(yv, [1,1,1]/3.)
         maxsig = max(yvsmooth)/yvstdev
         maxsigarr[b] = maxsig
         normzone = 10+indgen(ny-20)
         yvnorm = yv / max(yv[normzone])

         ;oplot, yv
         yoversample = [usey, usey-0.1, usey-0.2, usey-0.3, usey-0.4, usey+0.1, usey+0.2, usey+0.3, usey+0.4, usey+0.5]
         yoversample = yoversample[sort(yoversample)]
         yvoversample =    interpol(yvnorm[usey], usey, yoversample)  ; oversample by factor 10
         if b ne startb then begin
            if xc[b] lt xrange[0] then continue
            ; oplot, yoversample, yvoversample

              ; get relative lag versus last xblock - introduced pixel-rounding errors, so don't use
            ; lag = findgen(2*2*10)-2*10  ; maximum +/- 2 pixels
            ; lagpix = lag/10
            ; ccor = c_correlate(oldyvoversample, yvoversample, lag)
            ; w = where(ccor eq max(ccor))
            ; relshift = median(lagpix[w], /even)

              ; get absolute lag versus central xblock
            abslag = absshift*10 + findgen(2*2*10)-2*10  ; maximum +/- 2 pixels around last measurement
            lagpix = abslag/10
                ; ideally should limit it to within a few pix of the last solution
            ccor = c_correlate(centyvoversample, yvoversample, abslag)
            w = where(ccor eq max(ccor), ct)
            if ct eq 0 then begin
               print, '    WARNING: could not find cross-correlation peak at xc=', clip(xc[b])
               continue
            endif
            absshift = median(lagpix[w], /even)

            ; cumshift = cumshift + relshift
            ; cumshiftarr[b] = cumshift
            absshiftarr[b] = absshift
            ; print, b, '(', rclip(x0[b],4), '-', rclip(x1[b],4), ')   ', fpr(absshift,3.1), '  ', fpr(relshift,3.1), ' ', fpr(cumshift,3.1), '  ', fpr(maxsig,6.2)

            oplot, yoversample-absshift, yvoversample, $
                   color=pcol(255*(xc[b]-xrange[0])/rx, 255-255*abs(xc[b]-mean(xrange))/rx, 255.*(xrange[1]-xc[b])/rx, colors=colors) 
                   ;color=rgb(((startb-b) > 0)*10,255-abs(b-startb)*10,((-startb+b) > 0)*10)

         endif else begin
            centyv = yv
            centyvoversample = yvoversample
            absshift = 0.
            cumshift = 0.
            ; cumshiftarr[b] = 0.
            absshiftarr[b] = 0.
            oplot, yoversample, yvoversample, thick=2, color=pcol(255,255,0, colors=colors)
         endelse
         oldyv = yv
         oldyvoversample = yvoversample
      endfor
   endfor

   reltracearr = absshiftarr ; (absshiftarr + cumshiftarr)/2.
   minsig = (3.0 < median(maxsigarr)) > median(maxsigarr)*0.01
   isgoodtrace = (maxsigarr gt minsig) and finite(reltracearr)
      ;and abs(absshiftarr-cumshiftarr) lt 3.0 
   goodtrace = where(isgoodtrace, nfit)

   ; print, xc[goodtrace] 
   ; print, reltracearr[goodtrace]
   if nfit le tracefitorder then tracefitorder = nfit-1

   ; tracepar = polyiterfit(xc[goodtrace], reltracearr[goodtrace], order=tracefitorder,niter=2)   ; this should be updated
   par = [0]
   for o = 1, tracefitorder do begin      
      par = polyfitg(xc[goodtrace], reltracearr[goodtrace], o, guess=[par, 0], chi2=chi2) 
   endfor
   tracepar = par
   ; doesn't do any outlier rejection.


   tracey = poly(findgen(nx), tracepar) ; relative trace
   traceyxc = poly(xc, tracepar)


   ; plot 3: blocked image, showing the trace profile
   plot, [0], [0], xrange=[0,nx-1], yrange=[-10.5,10.5], /xsty, /ysty, /nodata, charsize=2
   for b = 0, nblock-1 do begin
     for y = ypeak[0]-10, ypeak[0]+10 do begin
        bv = (255*sqrt((blockim[b,y]/max(blockim)) > 0.) < 255.)
        polyfill, [x0[b], x1[b]+1, x1[b]+1, x0[b]], [y-0.5,y-0.5,y+0.5,y+0.5]-ypeak[0], color=pcol(bv,bv,bv, colors=colors, /inv), /data
     endfor
   endfor
   !p.multi[0] += 1
   plot, [0], [0], xrange=[0,nx-1], yrange=[-10.5,10.5], /xsty, /ysty, /nodata, /noerase, charsize=2
   for i = 0, n_elements(xc)-1 do $
      oplot, [xc[i]], [absshiftarr[i]], psym=isgoodtrace[i] ? 1 : 7, $
      color=pcol(255*(xc[i]-xrange[0])/rx, 255-255*abs(xc[i]-mean(xrange))/rx, 255.*(xrange[1]-xc[i])/rx, colors=colors), $ 
      thick=isgoodtrace[i] ? 3 : 1, symsize=2

   ; oplot, xc, cumshiftarr, psym=1, color=pcol(70,70,70, colors=colors)
   oplot, [xc[startb]], [0], psym=1, color=pcol(255,255,0, colors=colors), thick=3, symsize=3


   oplot, findgen(nx), tracey, color=pcol(255,0,255, colors=colors)
   !p.multi[0] -= 1

   endif


   ; now make a stacked profile, this time using the whole y-range
   profileoversample = fltarr(ny*10)
   yoversample = indgen(10*ny)/10.

   ; Plot 4 (2 if settrace): shifted profiles and final profile
   plot, yoversample, profileoversample, xrange=[5, (ny-10)], yrange=[min(blockim[*,5:ny-5]), max(blockim[*,5:ny-5])], /xsty, /ysty, charsize=2
   for b = 0, nblock-1 do begin
      yv = (blockim[b,*]) [*]  ; yv is the (unshifted) profile over this block
      yv -= median(yv, /even)
      shiftyvoversample = interpol(yv, indgen(ny), yoversample+traceyxc[b])
      oplot, indgen(ny)-traceyxc[b], yv, color=pcol(255*(xc[b])/nx, (255-255*abs(xc[b]-nx/2)/nx), 255.*(nx-xc[b])/nx, colors=colors)
      profileoversample += shiftyvoversample      
   endfor
   profileoversample /= nblock
   profile = interpol(profileoversample, yoversample, findgen(ny))
   
   oplot, profile, thick=3
 
   ; find sources here if we turn that on again

   if n_elements(plotfile) gt 0 then begin
      psclose
      retitleplot, plotfile, 'Trace/profile plots for '+filename
   endif

   if n_elements(profilefile) ge 1 then begin
      openw, 2, profilefile
      printf, 2, '# Profile file for '+clip(removepath(filename))
      printf, 2, '#XREF: ', clip(xstart)
      printf, 2, '#ypix profile'
      for y = 0, ny-1 do begin
         printf, 2, clip(y,5), profile[y]
      endfor
      close, 2
   endif 
   if n_elements(tracefile) ge 1 then begin
      openw, 2, tracefile
      printf, 2, '# Trace polynomial for '+clip(removepath(filename))
      str = ''
      for p = 0, n_elements(tracepar)-1 do str += clip(tracepar[p])+' '
      printf, 2, str
      close, 2
   endif 

   ;printf, 2, '# List of source positions (ypeak, dmin, dmax, peakval):'
   ;for i = 0, ct-1 do $
   ;  printf, 2, 'SOURCEY: ', yoversample[peaks[i]], -apm[i], app[i], peakval[i]
   ;printf, 2, '# Tracing polynomial, relative to XREF and SOURCEY above: ycent[s,x] = sourcey[s] + poly(x);  poly(xref)~=0'
   ;str = 'TRACEPOLY: '
   ;for p = 0, n_elements(tracepar)-1 do str += clip(tracepar[p])+' '
   ;printf, 2, str
   ;printf, 2, '# Spectral-summed profile at x=XREF (y, flux):'
   ;close, 2

   !p.multi = 0
   !p.position = 0
end


pro findsources1d
   ; Source finder - currently turned off
   findsources = 0
   if findsources then begin
   profilem = profile - median(profile,11)
     absdiff = median(abs(profilem-shift(profilem,1)))
     smoothprofile = convol(profileoversample,(1+fltarr(51))/51.) ; used for sourcefinding
     smoothprofile = smoothprofile - median(smoothprofile,201)
     oplot, yoversample, smoothprofile, color=pcol(255,0,0, colors=colors)

     peaks = where(profileoversample gt 0 and $
                 smoothprofile gt absdiff*7 and $
                 smoothprofile gt shift(smoothprofile,1) and smoothprofile ge shift(smoothprofile,-1) and $
                 smoothprofile gt shift(smoothprofile,2) and smoothprofile ge shift(smoothprofile,-2) and $
                 smoothprofile gt shift(smoothprofile,3) and smoothprofile ge shift(smoothprofile,-3), ct)
     peaky = fltarr(ct)
     apm = fltarr(ct)
     app = fltarr(ct)
     peakval = fltarr(ct)
     for i = 0, ct-1 do begin
        pi = peaks[i]
        peakyinit = yoversample[pi] ; really should maybe adjust this with a centroid fit...?
  
        peaky = getlinecenter( findgen(n_elements(profile)), profile, peakyinit, fitrad=7, unc=unc)
        ; peaky = peakyinit
  
        oplot, yoversample[[pi,pi]], [-1e5,1e6], color=pcol(255,255,0, colors=colors)
        app[*] = 0.
        apm[*] = 0.
        peakval[i] = profileoversample[pi]
        for dir = -1, 1, 2 do begin
          j = 0
          lastvalsmooth = smoothprofile[pi]
          while 1 do begin
            j += dir
            if pi+j le 0 then break
            if pi+j ge ny*10-1 then break
            if profileoversample[pi+j] lt peakval[i]*0.45 then begin  ; hwhm reached
               j *= 1.3  ; Ideally should fit a gaussian here or something?
               break
            endif
            if smoothprofile[pi+j] gt lastvalsmooth then break
          endwhile
          if dir eq -1 then apm[i] =-j/10. 
          if dir eq 1  then app[i] = j/10.
        endfor
        print, 'Source at ', fpr(peaky[i],3.1), ' (+', clip(fpr(app[i],3.1)), ' -', clip(fpr(apm[i],3.1))+'), peak flux ', clip(profileoversample[pi])
        oplot, yoversample[peaks[[i,i]]]-apm[i], [-1e5,1e6], color=pcol(255,255,0, colors=colors), linestyle=1
        oplot, yoversample[peaks[[i,i]]]+app[i], [-1e5,1e6], color=pcol(255,255,0, colors=colors), linestyle=1
     endfor
     print, ' All positions referenced at x = ', clip(xstart)
   endif
end
