pro writespec, filename, spec_in, h

  if n_elements(spec_in) eq 0 then begin
    print, 'No spectral data; not writing file to disk.'
  endif

  openw, 1, filename
  for i = 0, n_elements(h)-1 do begin
    printf, 1, '# '+h[i]
  endfor
  if n_elements(h) gt 0 then printf, 1, '# END'

  if size(spec_in, /tn) eq 'STRUCT' then begin
     spec = spec_in
  endif else begin
     nl = n_elements(spec_in) 
     for i = 0, n_elements(spec_in)-1 do printf, 1, spec_in[i]
     close, 1
     return
  endelse

  ; ideally should see if these member variables even exist
  wavyes = total(spec.wav) ne 0 and total(finite(spec.wav)) gt 0
  skyyes = total(spec.sky) ne 0 and total(finite(spec.sky)) gt 0
  uncyes = total(spec.unc) ne 0 and total(finite(spec.unc)) gt 0
  pixyes = total(spec.pix) ne 0 and total(finite(spec.pix)) gt 0
  if tag_exist(spec,'YPIX') then ypixyes = total(spec.ypix) ne 0 and total(finite(spec.ypix)) gt 0 else ypixyes = 0
  if tag_exist(spec,'RESP') then respyes = total(spec.resp) ne 0 and total(finite(spec.resp)) gt 0 else respyes = 0
  if wavyes eq 0 then begin
     ; pixel comes first if no wavelength
     cols=['xpixel', 'flux'] 
     if skyyes then cols = [cols, 'sky_flux']
     if uncyes then cols = [cols, 'flux_unc']
     if ypixyes then cols = [cols, 'ypixel']
     if respyes then cols = [cols,'response']
  endif else begin
     ; wavelength comes first otherwise
     cols=['wavelen','flux']
     if skyyes then cols = [cols, 'sky_flux']
     if uncyes then cols = [cols, 'flux_unc']
     if pixyes then cols = [cols, 'xpixel']
     if ypixyes then cols = [cols, 'ypixel']
     if respyes then cols = [cols,'response']
  endelse

  colheadstr = '## '
  collen = intarr(n_elements(cols))+12
  if pixyes then collen[where(cols eq 'xpixel')] = 7
  if ypixyes then collen[where(cols eq 'ypixel')] = 9
  collen[0] -= 3
  for c = 0, n_elements(cols)-1 do colheadstr += rclip(cols[c],collen[c])+' '
  printf, 1, colheadstr
  ; if respyes eq 0 then printf, 1, '## wavelength     flux         sky_flux     flux_unc     xpixel  ypixel'
  ; if respyes eq 1 then printf, 1, '## wavelength     flux         sky_flux     flux_unc     xpixel  ypixel  response'
  for i = 0, n_elements(spec)-1 do begin
    outstr = ''

    for c = 0, n_elements(cols)-1 do begin
       if cols[c] eq 'wavelen'  then outstr += rclip(spec[i].wav,12) + ' '
       if cols[c] eq 'flux'     then outstr += rclip(spec[i].flux,12) + ' '   ; if it's a double precision array this will produce a bad value
       if cols[c] eq 'sky_flux' then outstr += rclip(spec[i].sky,12)+' '
       if cols[c] eq 'flux_unc' then outstr += rclip(spec[i].unc,12)+' '
       if cols[c] eq 'xpixel'   then outstr += rclip(long(spec[i].pix),7)+' '
       if cols[c] eq 'ypixel'   then outstr += rclip(fpr(spec[i].ypix,5.3),9)+' '
       if cols[c] eq 'response' then outstr += rclip(spec[i].resp,12)+' '
    endfor
    printf, 1, outstr

    ; if respyes eq 0 then printf, 1, fpr(spec[i].wav,6.4) + string(spec[i].flux) + string(spec[i].sky) + string(spec[i].unc) + '  '+clip(long(spec[i].pix),7) + ' '+fpr(spec[i].ypix,5.3)
    ; if respyes eq 1 then printf, 1, fpr(spec[i].wav,6.4) + string(spec[i].flux) + string(spec[i].sky) + string(spec[i].unc) + '  '+clip(long(spec[i].pix),7) + ' '+fpr(spec[i].ypix,5.3) + string(spec[i].resp)
    ;if skyyes eq 0 and pixyes eq 0 then printf, 1, spec[i].wav, spec[i].flux
    ;if skyyes eq 1 and pixyes eq 0 then printf, 1, spec[i].wav, spec[i].flux, spec[i].sky
    ;if skyyes eq 1 and pixyes eq 1 then printf, 1, spec[i].wav, spec[i].flux, spec[i].sky, spec[i].unc
    ;if skyyes eq 1 and pixyes eq 1 then printf, 1, spec[i].wav, spec[i].flux, spec[i].sky, spec[i].pix
  endfor


  close, 1
end
