pro lrisconnect, bluefilename, redfilename, outfilename, plotfilename, displayoff=displayoff, useskythresh=useskythresh, setrescale=setrescale, rescaleside=rescaleside

      if n_elements(displayoff) eq 0 then displayoff = 0

      ; combine blue and red spectra with automatic rescaling.

      if n_elements(rescaleside) eq 0 then rescaleside = 'red' ; which half to actually adjust.  could allow geom. mean?

      if file_test(redfilename) eq 0 then return
      if file_test(bluefilename) eq 0 then return

      sb = readspec(bluefilename, hb)
      sr = readspec(redfilename, hr)

      if max(sb.wav) gt max(sr.wav) then begin  ; if blue and red are backward, reverse them
          lrisconnect, redfilename, bluefilename, outfilename, plotfilename
      endif

      hout = hb
      ; could instead make a list of here of parallel keywords to keep both versions of...
      sxaddpar, hout, 'INFILEB', removepath(bluefilename)
      sxaddpar, hout, 'INFILER', removepath(redfilename)
      sxaddpar, hout, 'EXPTIME', mean([sxpar(hb,'EXPTIME'),sxpar(hr,'EXPTIME')])
      sxaddpar, hout, 'TOTALEXP', mean([sxpar(hb,'TOTALEXP'),sxpar(hr,'TOTALEXP')])
      sxaddpar, hout, 'DATE_BEG', min([sxpar(hb,'DATE_BEG'),sxpar(hr,'DATE_END')])
      sxaddpar, hout, 'DATE_END', max([sxpar(hb,'DATE_BEG'),sxpar(hr,'DATE_END')])
      sxaddpar, hout, 'MINWAV', min([sxpar(hb,'MINWAV'),sxpar(hr,'MAXWAV')])
      sxaddpar, hout, 'MAXWAV', max([sxpar(hb,'MINWAV'),sxpar(hr,'MAXWAV')])

      target = repstr(sxpar(hb,'TARGNAME'),' ','')
      if strpos(target,'_S') gt 0 then target = strmid(target,0,strpos(target,'_S'))  ; offset stars
      if n_elements(outfilename) eq 0 then outfilename = clip(target)+'.spec'
      if n_elements(plotfilename) eq 0 then plotfilename = repstr(outfilename,'.spec','_rescale.eps')

      if n_elements(useskythresh) eq 0 then useskythresh = 150. ; counts/pix below which we also use the sky

      baprad = sxpar(hb,'APRAD')
      raprad = sxpar(hr,'APRAD')

      print, '  Connecting '+removepath(bluefilename)+' and '+removepath(redfilename), format='($,A)'
      if clip(sxpar(hr,'INSTRUME')) eq 'LRIS' then begin 
          sr = sr[0:n_elements(sr)-2] ; last pixels on red seem to be bad. (LRIS-R3 only, technically)
      endif


      ; determine the full region of overlap
      overlaprange = [min(sr.wav), max(sb.wav)]

      nooverlap = 0
      if overlaprange[0] gt overlaprange[1] then begin 
         ; If no overlap we can't accurately scale anything, so skip the whole scaling section.
         print
         print, '   No red/blue overlap!', format='($,A)'
         nooverlap = 1
      endif
      if sxpar(hb,'GRNAME') eq '300/5000' and sxpar(hb,'DICHNAME') eq '680' then begin
         print
         print, '   Cannot use rescaling algorithm for 300 grism due to second-order contamination.', format='($,A)'
         attachwav = 6500.
         nooverlap = 1
      endif
      if nooverlap eq 0 then begin

         ; Find the region of "optimum" overlap by calculating the product of response functions
         ;  and calculating the maximum +/- a bit

         woverblue = where(sb.wav ge overlaprange[0])
         bluerelrespover = sb[woverblue].resp / max(sb.resp)  ; blue response (normalized using the whole spec) for overlap pixels

         woverred =  where(sr.wav le overlaprange[1])
         redrelrespover =  sr[woverred].resp / max(sr[where(sr.wav lt overlaprange[1]+1000)].resp)
                                                 ; red response (normalized over the nearest 1000pix)

         ; interpolate sr[red] onto conv[blue] wavelengths
         wavover = sb[woverblue].wav
         redrelrespoveri = interpol(redrelrespover, sr[woverred].wav, wavover)

         ; heavily penalize pixels close to the red edge (where there are bias subtraction issues)
         w = where(wavover lt sr[30].wav, ct)
         if ct gt 0 then redrelrespoveri[w] *= 0.2

         ; calculate product of the two response functions
         respxresp = redrelrespoveri * bluerelrespover

         ; try to find a region with relatively high response on both sides

         ;wgoodrespxresp = where(respxresp gt max(respxresp)*0.2 and $
         ;                                        bluerelrespover gt 0.15 and redrelrespoveri gt 0.15,  ct)
         wgoodrespxresp = where(respxresp gt max(respxresp)*0.2 and $
                                                 bluerelrespover gt 0.11 and redrelrespoveri gt 0.11,  ct)
         if ct lt 40 then wgoodrespxresp = where(respxresp gt max(respxresp)*0.2 and $
                                                 bluerelrespover gt 0.07 and redrelrespoveri gt 0.07,  ct)
         if ct lt 30 then wgoodrespxresp = where(respxresp gt max(respxresp)*0.2 and $
                                                 bluerelrespover gt 0.03 and redrelrespoveri gt 0.03,  ct)

         ; older method used a narrower region
         ;wgoodrespxresp = where(respxresp gt max(respxresp)*0.25 and $
         ;                                        bluerelrespover gt 0.15 and redrelrespoveri gt 0.15,  ct)
         ;if ct lt 45 then wgoodrespxresp = where(respxresp gt max(respxresp)*0.25 and $
         ;                                        bluerelrespover gt 0.11 and redrelrespoveri gt 0.11,  ct)
         ;if ct lt 40 then wgoodrespxresp = where(respxresp gt max(respxresp)*0.25 and $
         ;                                        bluerelrespover gt 0.07 and redrelrespoveri gt 0.07,  ct)
         ;if ct lt 30 then wgoodrespxresp = where(respxresp gt max(respxresp)*0.25 and $
         ;                                        bluerelrespover gt 0.03 and redrelrespoveri gt 0.03,  ct)

         if ct lt 10 then begin              
            print
            print, '   Insufficient blue/red overlap!  (Only '+$
                   fpr(overlaprange[1]-overlaprange[0],2.1)+' Angstrom)', format='($,A)'
            nooverlap = 1
         endif else begin
            useoverlaprange = [min(wavover[wgoodrespxresp]), max(wavover[wgoodrespxresp])] ; rescale calculation region

            ; useoverlapoptimum = (wavover[where(respxresp eq max(respxresp))]) [0]          ; connection point - old method
            ; new method - just check which is higher.
            higherresp = (bluerelrespover gt redrelrespoveri)  ; which relative response is higher?  1=blue, 0=red
            useoverlapoptimumw = where(higherresp ne shift(higherresp,-1), ct) ; will generally return two, first is the desired one
            if ct gt 0 then useoverlapoptimum = wavover(useoverlapoptimumw[0])
            if ct eq 0 and higherresp[0] eq 1 then useoverlapoptimum = useoverlaprange[1]
            if ct eq 0 and higherresp[0] eq 0 then useoverlapoptimum = useoverlaprange[0]
            useoverlapwidth = useoverlaprange[1]-useoverlaprange[0]
            nooverlap = 0
         endelse

         if nooverlap eq 0 and n_elements(setrescale) eq 0 then begin
            if useoverlapwidth lt 45. then begin
               print
               print, '   Caution: marginal blue/red overlap ('+fpr(useoverlapwidth,2.1)+' Ang):', format='($,A)'
            endif
            ; print, ' '+clip(useoverlaprange[0])+'-'+clip(useoverlaprange[1])+' ', format='($,A)'

            ;window, 0
            ;!p.multi = 0
            ;!p.position = 0
            ;plot, wavover, wavover*0, /nodata, /xsty, yrange=[0,1]
            ; oplot, wavover, redrelrespoveri, color=rgb(255,0,0)
            ; oplot, wavover, bluerelrespover, color=rgb(0,0,255)
            ; oplot, wavover, respxresp/max(respxresp), color=rgb(255,255,0)
            ;stop

            srdeline = fltarr(n_elements(sr))
            sbdeline = fltarr(n_elements(sb))
            if sxpar(hb,'GRNAME') eq '600/4000' then begin
               wrline = where(sr.wav gt 5569 and sr.wav lt 5590, ct)  ; line is at 5577.346680
               if ct gt 0 then srdeline[wrline] = !values.f_nan ; avoid the bright night sky line
               wbline = where(sb.wav gt 5569 and sb.wav gt 5590, ct)
               if ct gt 0 then sbdeline[wbline] = !values.f_nan ; avoid the bright night sky line
            endif

            bdwav = sb.wav - shift(sb.wav, 1)
            bdwav[0] = bdwav[1]            
            bozone = where(sb.wav ge useoverlaprange[0] and sb.wav le useoverlaprange[1] and finite(sb.flux+sb.sky+sbdeline), ctb)

            rdwav = sr.wav - shift(sr.wav,1)
            rdwav[0] = rdwav[1]
            rozone = where(sr.wav ge useoverlaprange[0] and sr.wav le useoverlaprange[1] and finite(sr.flux+sr.sky+srdeline), ctr)

            ; average S/N per pix in overlap region
            avovsnblue = median(sb[bozone].flux/sb[bozone].unc, /even)
            avovsnred  = median(sr[rozone].flux/sr[rozone].unc, /even)

            ; average counts in overlap region
            avovctblue = median(sb[bozone].flux*sb[bozone].resp)*sxpar(hb,'EXPTIME')
            avovctred  = median(sr[rozone].flux*sr[rozone].resp)*sxpar(hr,'EXPTIME')

            skyw = 0  ; can't be trusted as a general tool
            ;if avovctblue lt useskythresh or avovctred lt useskythresh then $
            ;   skyw = 1 else skyw = 0

            if avovsnblue lt 3. or avovsnred lt 3. then begin
               print
               print, '  Warning: Low S/N in overlap region - blue :', clip(fpr(avovsnblue,7.2)), ',  red :', clip(fpr(avovsnred,7.2))
            end

            if ctb eq 0 or avovsnblue lt 1.3 then begin
               bonorm = -1
            endif else begin
               bonorm = total((sb[bozone].flux+sb[bozone].sky*skyw) * bdwav[bozone]) / total(bdwav[bozone])
            endelse
  
            if ctr eq 0 or avovsnred lt 1.3 then begin
               ronorm = -1
            endif else begin
               ronorm = total((sr[rozone].flux+sr[rozone].sky*skyw*baprad/raprad) * rdwav[rozone]) / total(rdwav[rozone])
            endelse

            if bonorm ne -1 and ronorm ne -1 then begin
               rescale = bonorm/ronorm
               if bonorm lt 0 then rescale = 0.
               if ronorm lt 0 then rescale = 0.
               if keyword_set(noscale) then rescale = 1.0 ; should automatically recognize really faint stuff

               if rescale lt 0.1 or rescale gt 10.0 or rescale le 0 then begin
                  print
                  if rescale gt 0 then print, '  WARNING - Red/blue rescaling factor is VERY large ('+clip(rescale)+').'
                  if rescale le 0 then print, '  ERROR   - Red/blue rescale value could not be calculated or is negative.'
                  print, '            This probably means traces of different objects are being matched or'
                  print, '            that there is a problem with tracing or flux calibration.'
                  print, '            Check the individual files.'
                  for s = 0, 1 do begin
                     if s eq 0 then hs = hb else hs = hr
                     ; sort of pointless now.
                     ;for cf = 0, 6 do begin
                     ;   filec = removepath(sxpar(hs, 'INFILE'+clip(cf), count=ct))
                     ;   if ct eq 0 then filec = removepath(sxpar(hs, 'COMB'+clip(cf), count=ct))
                     ;   expc = sxpar(hs, 'EXPTIME'+clip(cf), count=ct)
                     ;   if ct eq 0 then expc = sxpar(hs, 'EXPT'+clip(cf), count=ct)
                     ;   if ct gt 0 then print, '             ', filec, ' ', clip(expc)+'s'
                     ;endfor
                  endfor
                  if rescale lt 0.05 or rescale gt 20.0 then return  ; skip it entirely
               endif else begin  
                 if rescale lt 0.5 or rescale gt 2.0 then begin
                    print
                    print, '  WARNING - Red/blue rescaling factor is large ('+clip(rescale)+')', format='($,A)'
                 endif               
                 if rescale lt 0.0 then begin
                    ; this cannot currently happen
                    print
                    print, '  WARNING - Rescaling factor is negative!!   Not rescaling at all.'
                    rescale = 1.0
                 endif
              endelse
            endif else begin ; rescale calculation bock
               print, '  Unable to measure rescale factor for blue/red spectra; not scaling.', format='($,A)'
               rescale = 1.0
            endelse

         endif else begin ; not enough overlap
            if n_elements(setrescale) gt 0 then rescale = setrescale else $
            rescale = 1.0   ; if no overlap
         endelse

      endif else begin
         rescale = 1.0
      endelse

      ; actually do the rescaling

      oldsr = sr
      oldsb = sb
      if rescaleside ne 'blue' then begin
           print, '(x'+clip(clip(rescale,5))+') ', format='($,A)'
           sr.flux *= rescale ; the red, not the blue, is rescaled.  should reconsider this, since red has less airmass/seeing dependence.
           if tag_exist(sr, 'sky') then sr.sky *= rescale ; should we aperture-scale this too?
           if tag_exist(sr, 'unc') then sr.unc *= rescale
           if tag_exist(sr, 'resp') then sr.resp /= rescale
      endif 
      if rescaleside ne 'red' then begin
           rescale = 1./rescale
           print, ' (blue x'+clip(clip(rescale,5))+') ', format='($,A)'
           sb.flux *= rescale ; the red, not the blue, is rescaled.  should reconsider this, since red has less airmass/seeing dependence.
           if tag_exist(sb, 'sky') then sb.sky *= rescale ; should we aperture-scale this too?
           if tag_exist(sb, 'unc') then sb.unc *= rescale
           if tag_exist(sb, 'resp') then sb.resp /= rescale
      endif

      makeplot = 1
      if makeplot then begin
        if n_elements(bozone) eq 0 then $
           bozone = where(sb.wav ge min(sr.wav) and sb.wav le max(sb.wav) and finite(sb.flux+sb.sky), ct)
        if n_elements(rozone) eq 0 then $
           rozone = where(sr.wav ge min(sr.wav) and sr.wav le max(sb.wav) and finite(sr.flux+sr.sky), ct)

        ; plot various things to the screen and to a ps file
        for output = displayoff, 1 do begin
           !p.multi = [0,2,2]
           if output eq 0 then begin
              errorstatus = 0
              catch, errorstatus
              if errorstatus ne 0 then begin
                 print, ' X-windows display not working: turning off display'
                 displayoff = 1
                 catch, /cancel
                 continue
              endif else begin
                 window, 0, xsize=950, ysize=400
              endelse
              catch, /cancel
              
              title = ''
              oldbcol = rgb(55,45,128)
              newbcol = rgb(95,145,255)
              oldrcol = rgb(128,0,0)
              newrcol = rgb(255,128,0)
              gray = rgb(128,128,128)
              green = rgb(0,255,0)
           endif
           if output eq 1 then begin
              psopen, plotfilename, /encaps, xsize=9.5, ysize=4, /inches
              device, /color
              !p.font=0
              title = '!17'
              device, /times, font_index=17
              colors = transpose([[0,0,0],$
                                  [0,0,200],$ 
                                  [100,50,50],$ 
                                  [255,0,0],$
                                  [128,128,128],$
                                  [0,128,0],$
                                  [50,50,100]])
              tvlct, colors
              oldbcol = 6
              newbcol = 1
              oldrcol = 2
              newrcol = 3 
              gray = 4
              green = 5
           endif
           for px = 0, 3 do begin  ; plot-x
              if px eq 0 then !p.position = [0.09,0.4,0.39,0.98]
              if px eq 1 then !p.position = [0.39,0.4,0.69,0.98]
              if px eq 2 then !p.position = [0.69,0.4,0.99,0.98]
              if px eq 3 then !p.position = [0.09,0.1,0.99,0.4]
              if n_elements(useoverlaprange) eq 2 then $
                xrange=useoverlaprange else $
                continue  ; temporary solution
              yrange=[0,1.05*max([sb[bozone].flux+sb[bozone].sky,(sr[rozone].flux+sr[rozone].sky)*baprad/raprad])]
              if px eq 3 then xrange=useoverlaprange+[-1000,+1000]
              if px eq 3 then yrange=[0,2*max([median(oldsr[rozone].flux),median(sr[rozone].flux),median(sb[bozone].flux)])] ;[,,
              if px eq 1 or px eq 2 then ytickformat='(A1)' else delvarx, ytickformat
              plot, [0], [0], xrange=xrange, yrange=yrange, /xsty, /ysty, charsize=0.75, title=title, ytickformat=ytickformat
                 if px eq 0 then xyouts, xrange[0]+5, yrange[1]*0.9, 'obj ('+clip(target)+')'
                 if px eq 1 then xyouts, xrange[0]+5, yrange[1]*0.9, 'sky'
                 if px eq 2 then xyouts, xrange[0]+5, yrange[1]*0.9, 'sum'
                 if px eq 3 then xyouts, xrange[0]+(xrange[1]-xrange[0])*0.05, yrange[1]*0.7, 'obj ('+clip(target)+')'
              ;oplot, [5577,5577], yrange, linestyle=1
              y0 = (px mod 3 ne 1)*oldsb.flux+(px mod 3 ne 0)*oldsb.sky*baprad/raprad ; sky is always matched to red, I guess
              y1 = (px mod 3 ne 1)*sb.flux   +(px mod 3 ne 0)*sb.sky*baprad/raprad
              y2 = (px mod 3 ne 1)*oldsr.flux+(px mod 3 ne 0)*oldsr.sky*baprad/raprad
              y3 = (px mod 3 ne 1)*sr.flux   +(px mod 3 ne 0)*sr.sky*baprad/raprad
              if px eq 3 then y0 = convol(y0,(1+fltarr(7))/7., /edge_truncate)
              if px eq 3 then y1 = convol(y1,(1+fltarr(7))/7., /edge_truncate)
              if px eq 3 then y2 = convol(y2,(1+fltarr(7))/7., /edge_truncate)
              if px eq 3 then y3 = convol(y3,(1+fltarr(7))/7., /edge_truncate)
              oplot, oldsb.wav, y0, color=oldbcol, psym=10
              oplot, sb.wav, y1, color=newbcol, psym=10
              oplot, oldsr.wav, y2, color=oldrcol, psym=10
              oplot, sr.wav, y3, color=newrcol, psym=10
              if px eq 0 then oplot, wavover, yrange[1]*bluerelrespover, color=gray
              if px eq 0 then oplot, wavover, yrange[1]*redrelrespoveri, color=gray
              if px eq 3 then oplot, sb.wav, yrange[1]*sb.resp/max(sb.resp), color=gray
              if px eq 3 then oplot, sr.wav, yrange[1]*sr.resp/max(sr[where(sr.wav lt useoverlaprange[1]+1000)].resp), color=gray
              if px eq 0 or px eq 3 then for i = 0, 1 do oplot, useoverlaprange[i]*[1,1], yrange, thick=2
              if n_elements(useoverlapoptimum) gt 0 then oplot, useoverlapoptimum+[0,0], yrange, color=green, linestyle=2, thick=2
           endfor ; plot panel loop
           if output eq 1 then psclose
        endfor ; plot output loop
      endif

      ;print, ' @ '+overlapcenter, format='($,A)'
      ;if n_elements(attachwav) lt 0 then delvarx, attachwav

      if n_elements(useoverlapoptimum) gt 0 then attachwav = useoverlapoptimum
      if n_elements(attachwav) eq 0 then attachwav = strnum(sxpar(hb,'DICHNAME'))*10.  

      smerge = connectspec(sb, sr, attachwav=attachwav)
      if rescaleside ne 'blue' then sxaddpar, hout, 'RESCALER', rescale else $
      if rescaleside ne 'red'  then sxaddpar, hout, 'RESCALEB', rescale
      if n_elements(useoverlaprange) gt 0 then sxaddpar, hout, 'WAVOVER', clip(useoverlaprange[0])+'-'+clip(useoverlaprange[1])
      if n_elements(attachwav) gt 0 then sxaddpar, hout, 'ATCHWAV', attachwav



      ; rename some keywords to be instrument-specific
      if sxpar(hb,'INSTRUME') eq 'LRISBLUE' then begin
         blueexp = sxpar(hb,'TOTALEXP')
         if blueexp eq 0 then blueexp = sxpar(hb,'EXPTIME')
         sxaddpar, hout, 'BLUDBEG', sxpar(hb,'DATE_BEG')
         sxaddpar, hout, 'BLUDEND', sxpar(hb,'DATE_END')
         sxaddpar, hout, 'BLUNEXP', sxpar(hb,'NSUM')
         sxaddpar, hout, 'BLUEXPTM', blueexp
         for cf = 6, 0, -1 do begin
            expc = sxpar(hb, 'EXPTIME'+clip(cf), count=ct)
            if ct eq 0 then expc = sxpar(hb, 'EXPT'+clip(cf), count=ct)
            if ct gt 0 then sxaddpar, hout, 'BLUEXP'+clip(cf), expc, after='BLUEXPTM'
         endfor
         sxaddpar, hout, 'BLUFILT', sxpar(hb,'BLUFILT')
         sxaddpar, hout, 'GRISNAME', sxpar(hb,'GRISNAME')
         sxaddpar, hout, 'BLUBIN', sxpar(hb,'BINNING')
         sxaddpar, hout, 'BLUAPRAD', sxpar(hb,'APRAD')
         sxaddpar, hout, 'BLUBKDST', max([sxpar(hb,'BAKDIST'),sxpar(hb,'BAKDISTM'),sxpar(hb,'BAKDISTP')])
         sxaddpar, hout, 'BLUBKWID', max([sxpar(hb,'BAKWIDTH'),sxpar(hb,'BAKWIDTM'),sxpar(hb,'BAKWIDTP')])
         sxaddpar, hout, 'BLUARCF', removepath(sxpar(hb,'ARCFILE'))
         sxaddpar, hout, 'BLUCWAV', sxpar(hb,'CWAV')
         sxaddpar, hout, 'BLUDISP', sxpar(hb,'DISPAV')
         sxaddpar, hout, 'BLUSOL', removepath(sxpar(hb,'SOLFILE'))
         sxaddpar, hout, 'BLUFCAL1', sxpar(hb,'FLUXCAL1')
         if clip(sxpar(hb,'BLUFCAL2')) ne '0' then sxaddpar, hout, 'BLUFCAL2', sxpar(hb,'FLUXCAL2')
         sxaddpar, hout, 'BLUFAIR1', sxpar(hb,'FLUXAIR1')
         if clip(sxpar(hb,'BLUFAIR2')) ne '0' then sxaddpar, hout, 'BLUFAIR2', sxpar(hb,'FLUXAIR2')
         sxaddpar, hout, 'BLUFNAM1', sxpar(hb,'FLUXNAM1')
         if clip(sxpar(hb,'BLUFNAM2')) ne '0' then sxaddpar, hout, 'BLUFNAM2', sxpar(hb,'FLUXNAM2')
         maxifkeyb = ''
         for c = 0, n_elements(hb)-1 do begin
            nam = clip(strmid(hb[c],0,strpos(hb[c],'=')))
            if strmid(nam,0,6) eq 'INFILE' or strmid(nam,0,4) eq 'COMB' or $
               strmid(nam,0,5) eq 'SFILE' or strmid(nam,0,6) eq 'CFILE' then begin
                val = sxpar(hb, nam, /silent)
                cf = clip(strnum(nam))
                if strmid(nam,0,6) eq 'INFILE' or strmid(nam,0,5) eq 'SFILE' then sxaddpar, hout, 'SFILEB'+clip(cf), val
                if strmid(nam,0,4) eq 'COMB' or strmid(nam,0,6) eq 'CFILE'   then sxaddpar, hout, 'CFILEB'+clip(cf), val
            endif
         endfor
      endif
      if clip(sxpar(hr,'INSTRUME')) eq 'LRIS' then begin
         redexp = sxpar(hr,'TOTALEXP')
         if redexp eq 0 then redexp = sxpar(hr,'EXPTIME')
         sxaddpar, hout, 'REDEXPTM', redexp, after='BLUEXPTM'
         for cf = 6, 0, -1 do begin
            expc = sxpar(hr, 'EXPTIME'+clip(cf), count=ct)
            if ct eq 0 then expc = sxpar(hr, 'EXPT'+clip(cf), count=ct)
            if ct gt 0 then sxaddpar, hout, 'REDEXP'+clip(cf), expc, after='REDEXPTM'
         endfor
         sxaddpar, hout, 'REDDBEG', sxpar(hr,'DATE_BEG'), after='BLUDBEG'
         sxaddpar, hout, 'REDDEND', sxpar(hr,'DATE_END'), after='BLUDEND'
         sxaddpar, hout, 'REDNEXP', sxpar(hr,'NSUM'), after='BLUNEXP'
         sxaddpar, hout, 'REDFILT', sxpar(hr,'REDFILT'), after='BLUFILT'
         sxaddpar, hout, 'GRANAME', sxpar(hr,'GRNAME'), after='GRISNAME' ; can restore this back to graname or remove later
         sxaddpar, hout, 'REDBIN', sxpar(hr,'BINNING'), after='BLUBIN'
         sxaddpar, hout, 'REDAPRAD', sxpar(hr,'APRAD'), after='BLUAPRAD'
         sxaddpar, hout, 'REDBKDST', sxpar(hr,'BAKDIST'), after='BLUBKDST'
         sxaddpar, hout, 'REDBKWID', max([sxpar(hr,'BAKWIDTH'),sxpar(hr,'BAKWIDTM'),sxpar(hr,'BAKWIDTP')]), after='BLUBKWID'
         sxaddpar, hout, 'REDARCF', removepath(sxpar(hr,'ARCFILE')), after='BLUARCF'
         sxaddpar, hout, 'REDCWAV', sxpar(hr,'CWAV'), after='BLUCWAV'
         sxaddpar, hout, 'REDDISP', sxpar(hr,'DISPAV'), after='BLUDISP'
         sxaddpar, hout, 'REDSOL', removepath(sxpar(hr,'SOLFILE')), after='BLUSOL'
         sxaddpar, hout, 'REDFCAL1', sxpar(hr,'FLUXCAL1'), after='BLUFCAL1'
         if clip(sxpar(hr,'REDCAL2')) ne '0' then sxaddpar, hout, 'REDFCAL2', sxpar(hr,'FLUXCAL2'), after='BLUFCAL2'
         sxaddpar, hout, 'REDFAIR1', sxpar(hr,'FLUXAIR1'), after='BLUFAIR1'
         if clip(sxpar(hr,'REDFAIR2')) ne '0' then sxaddpar, hout, 'REDFAIR2', sxpar(hr,'FLUXAIR2'), after='BLUFAIR2'
         sxaddpar, hout, 'REDFNAM1', sxpar(hr,'FLUXNAM1'), after='BLUFNAM1'
         if clip(sxpar(hr,'REDFNAM2')) ne '0' then sxaddpar, hout, 'REDFNAM2', sxpar(hr,'FLUXNAM2'), after='BLUFNAM2'
         for c = 0, n_elements(hr)-1 do begin
            nam = clip(strmid(hr[c],0,strpos(hr[c],'=')))
            if clip(nam) eq '0' or clip(nam) eq '' or clip(nam) eq '#' then continue
            val = sxpar(hr, nam, /silent)
            if strmid(nam,0,6) eq 'INFILE' or strmid(nam,0,4) eq 'COMB' or $
               strmid(nam,0,5) eq 'SFILE' or strmid(nam,0,6) eq 'CFILE' then begin
                cf = clip(strnum(nam))
                if strmid(nam,0,6) eq 'INFILE' or strmid(nam,0,5) eq 'SFILE' then sxaddpar, hout, 'SFILER'+clip(cf), val
                if strmid(nam,0,4) eq 'COMB' or strmid(nam,0,6) eq 'CFILE'   then sxaddpar, hout, 'CFILER'+clip(cf), val
            endif else begin
              ; add any keywords in the input red header that aren't in the blue header
              outval = sxpar(hout,nam,count=ct)
              if ct eq 0 then $
                sxaddpar, hout, nam, val
           endelse
         endfor
      endif
 

      sxaddpar, hout, 'NSATURAT', sxpar(hr,'NSATURAT')+sxpar(hb,'NSATURAT')
      sxaddpar, hout, 'NWAV', n_elements(smerge)

      sxdelpar, hout, 'GRNAME' ; no longer relevant for combining both sides
      sxdelpar, hout, 'ELAPTIME'
      sxaddpar, hout, 'INSTRUME', 'LRIS+LRISBLUE'
      sxdelpar, hout, 'CAMERA'
      sxdelpar, hout, 'ARCFILE'
      sxdelpar, hout, 'CWAV'
      sxdelpar, hout, 'DISPAV'
      sxdelpar, hout, 'BINNING'
      sxdelpar, hout, 'FLATTYPE'

      sxdelpar, hout, 'FLUXCAL1'
      sxdelpar, hout, 'FLUXCAL2'
      sxdelpar, hout, 'FLUXAIR1'
      sxdelpar, hout, 'FLUXAIR2'
      sxdelpar, hout, 'FLUXNAM1'
      sxdelpar, hout, 'FLUXNAM2'
      ; could delete lots of other things associated with one file or another (and replace with averages / pair terms...)?

      delkeys = ['LTV1','LTV2','XBIN','YBIN','PXSCALE','DICHSEQF','RFLTSEQF','GRATSEQF','GRANSEQF', $
                 'COMB'+clip(indgen(10)), 'EXPT'+clip(indgen(10)), 'WINDOW', 'PANE', 'NAXIS1','NAXIS2',$
                 'YEXTRACT','CWAVE','NSUM','SATURATE','DISPERSN','APRAD','APSHARP','NSATURAT','NWAV',$
                 'WAVORDER','WAVNUDGE','BAKDISTM','BAKDISTP','BAKWIDTM','BAKWIDTP']
      for d = 0, n_elements(delkeys)-1 do sxdelpar, hout, delkeys[d]


      if n_elements(setrescale) gt 0 then print, '(x'+clip(clip(rescale,5))+') ', format='($,A)'
      print
      if nooverlap eq 0 and n_elements(skyw) gt 0 then if skyw then begin
          print, '    Faint source ('+clip(avovctblue)+'/'+clip(avovctred)+' ct/px in overlap region): using the sky. '
      endif
      print, '             -> ', removepath(outfilename)
 
      writespec, outfilename, smerge, hout

end
   
