pro lriscorrectheaders, files, check=check, maxdt=maxdt, dir=dir

  ; An updated version of correctlrisheaders.

  ; Much more thorough - always checks all keywords and makes no assumptions about certain keys
  ; being blocked together.

  ; look for (and replace) these keywords if missing
  checkkey = ['TARGNAME','TELESCOP','TELFOCUS','TUBETEMP','DOMEPOSN','FLIMAGIN','FLSPECTR','DRA','DDEC',$
'DTRACK','ADCNEED','CURRINST','ADCPRENA','RA','ROTMODE','ROTPOSN','ROTPPOSN','SECFOCUS','SECTHETX',$
'SECTHETY','SIMULATE','AIRMASS','AZ','DEC','EL','EQUINOX','HA','PONAME','MJD-OBS','ST','DATE-OBS',$
'SLITNAME','WAVELEN','REDFILT','BLUFILT']


  if n_elements(dir) eq 0 then dir = ''
  if n_elements(files) eq 0 then files = [file_search(dir+'b*_????.fits'),file_search(dir+'r*_????.fits')]
  if n_elements(files) eq 1 then if strpos(files,'*') ge 0 or strpos(files,'?') ge 0 then files = file_search(files)
  notblank = where(files ne '', ct)
  if ct eq 0 then return else files = files[notblank]
  nfiles = n_elements(files)

  if n_elements(maxdt) eq 0 then maxdt = 900.

  ; this structure is used only in matching, not for recognizing or filling in missing data.
  lris = replicate({filename:'', ut:'', utf:0.D, object:'', camera:'', $
                     dec:'---', ra:'---', targname:'---', misskeys:'', nmisskey:0},nfiles)

  for f = 0, n_elements(files)-1 do begin
     lris[f].filename = files[f]
     h = headfits(files[f], /silent)
     lris[f].ut = sxpar(h,'UTC')
     lris[f].utf = ten(sxpar(h,'UTC'))
     lris[f].object = sxpar(h,'OBJECT',/silent)
     lris[f].camera = strmid(files[f],0,1)

     v = sxpar(h,'TARGNAME', count=ct)
     if ct ge 1 then lris[f].targname = v
     v = sxpar(h,'RA', count=ct)
     if ct ge 1 then lris[f].ra = v
     v = sxpar(h,'DEC', count=ct)
     if ct ge 1 then lris[f].dec = v

     for k = 0, n_elements(checkkey)-1 do begin
        v = sxpar(h,checkkey[k],count=ct, /silent)
        if ct eq 0 then lris[f].nmisskey += 1
        if ct eq 0 then lris[f].misskeys += checkkey[k]+','
     endfor
  endfor

  for i = 0, 1 do begin
      if i eq 0 then begin
         incam = 'r'
         comparecam = 'b'
      endif else begin
         incam = 'b'
         comparecam = 'r'
      endelse 
      ch = lris[where(lris.camera eq incam)]        ; files to examine
      comp = lris[where(lris.camera eq comparecam)]  ; files to compare against

      w = where(ch.nmisskey eq 0, ctcomplete)
      if ctcomplete eq 0 then begin
         print, 'All LRIS-'+strupcase(incam)+' files are missing a keyword!'
      endif

     for f = 0, n_elements(ch)-1 do begin
        if ch[f].nmisskey gt 0 then begin
           dt = (ch[f].utf - comp.utf)*3600.D
           match = (where(abs(dt) eq min(abs(dt)) )) [0]
           mh = headfits(comp[match].filename)

           if abs(dt[match]) ge maxdt then begin
              print, ch[f].filename,        '['+clip(ch[f].object)+']:' $
                     + '  cannot find a match within '+clipzero(maxdt)+' sec.'
              continue
           endif
           timewarn = ''
           if abs(dt[match]) gt 90 then timewarn = ' (times differ significantly)
           print, ch[f].filename,        '['+clip(ch[f].object)+'] matched by ', $
                 comp[match].filename, '['+clip(comp[match].object)+'] ', $
                 ch[f].ut+'-'+comp[match].ut+' = '+clip(dt[match])+'s ' + timewarn
 
           h = headfits(ch[f].filename)
           inhnote = 'Inherited from '+removepath(comp[match].filename)
 
           misskeyarr = strsplit(ch[f].misskeys,',',/extract)
           for j = 0, n_elements(misskeyarr)-1 do begin
              matchval = sxpar(mh, misskeyarr[j], count=ct, /silent)
              if ct eq 0 then begin
                 print, '   '+clip(misskeyarr[j])+' is missing in match also!' 
              endif else begin
                 print, '   '+clip(misskeyarr[j])+' --> '+clip(matchval)
                 sxaddpar, h, misskeyarr[j], matchval, inhnote
              endelse

           endfor

           if keyword_set(check) eq 0 then $
              modfits, ch[f].filename, 0, h, exten=0
 
        endif
     endfor
  endfor







end
