pro lrisplottrace, skysubfile, specfile, ycenter=ycenter, ysize=ysize, outx0=outx0, outy0=outy0, xbin=xbin, ybin=ybin

   ; should common block these hardcode values shared with the other prog

   if n_elements(outx0) eq 0 then outx0 = 0
   if n_elements(outx0) eq 0 then outy0 = 0

   im = readfits(skysubfile, hi, /silent)
   nx = (size(im)) [1]
   ny = (size(im)) [2]

   sp = readspec(specfile, hs)

   target = sxpar(hi,'TARGNAME')
   gra = sxpar(hi,'GRNAME')
   cwav = sxpar(hi,'CWAVE') ; it's CWAV in the 2D spec...
   upos = strpos(target,'_')
   if upos gt 3 then target = strmid(target,0,upos)

   inxbin = sxpar(hi, 'XBIN')
   inybin = sxpar(hi, 'YBIN')
   xrebin = xbin*1./inxbin   ; rebinning factor
   yrebin = ybin*1./inybin

   if n_elements(ycenter) eq 0 then ycent = round(median(sp.ypix,/even)) else ycent = ycenter

   fitsx0 = 0
   fitsy0 = ycent - ysize*(ybin*1./inybin)/2
   fitsy1 = fitsy0  + ysize*(ybin*1./inybin)   -1

   ; crop y
   if ycent-ysize/2 lt 0 then begin
      newim = fltarr(nx, ny-fitsy0) + !values.f_nan
      newim[*,-fitsy0:*] = im
      im = newim[*,0:ysize-1]
   endif else begin
      if fitsy1 gt ny-1 then begin
         newim = fltarr(nx, ny+ceil(fitsy1-ny)+1) + !values.f_nan
         newim[*,0:ny-1] = im
         im = newim[*,fitsy0:fitsy1]
      endif else begin
         ; normal behavior when the trace is not near an edge
         im = im[*,fitsy0:fitsy1]
      endelse
   endelse

   ; bin
   im = im[0:fix(nx/xrebin)*xrebin-1,*] ; make sure input array size is a multiple of xbin (4 red, 2 blue)
   binim =  rebin(im, fix(nx/xrebin), fix((fitsy1-fitsy0+1)/yrebin))  ; does seem to preserve S/N...
   ;binim = srebin(im, xrebin, yrebin, /average)

   ; draw background color
   polyfill, [outx0-1-1,outx0-1-1,outx0+1024+1+1,outx0+1024+1+1], [outy0-1-1,outy0+ysize+1+1,outy0+ysize+1+1,outy0-1-1], color=rgb(100,100,100), /device

   ; scale and display
   scaleim = (binim - median(binim)) > (-10.)
   scaleim = 255 * ((scaleim / max([100., percentile(scaleim,98)])) < 1)
   tvscl, scaleim, outx0, outy0, /nan
   
   ; overplot trace
   aprad = sxpar(hs,'APRAD')
   ;traceybin = rebin(sp.ypix-fitsy0, fix(nx/xrebin))/yrebin 
   ;traceybinp = traceybin + aprad/yrebin
   ;traceybinm = traceybin - aprad/yrebin
   tracexdisp = (1.*sp.pix-fitsx0)/xrebin
   traceydisp = (1.*sp.ypix-fitsy0)/yrebin
   traceydispp = (sp.ypix+aprad-fitsy0)/yrebin
   traceydispm = (sp.ypix-aprad-fitsy0)/yrebin
   
   ; I don't really understand the need for the 1+, but the plotted lines do seem to be off otherwise (is it really bin+...?)
   w = where(sp.pix ge 0 and sp.wav gt 2900. and traceydisp gt 0 and traceydisp lt ysize, ct)
   if ct gt 0 then plots, outx0+tracexdisp[w], 1+outy0+traceydisp[w] , color=rgb(0,255,0), /device, linestyle=2
   w = where(sp.pix ge 0 and sp.wav gt 2900. and traceydispp gt 0 and traceydispp lt ysize, ct)
   if ct gt 0 then plots, outx0+tracexdisp[w], 1+outy0+traceydispp[w], color=rgb(0,255,0), /device, linestyle=0
   w = where(sp.pix ge 0 and sp.wav gt 2900. and traceydispm gt 0 and traceydispm lt ysize, ct)
   if ct gt 0 then plots, outx0+tracexdisp[w], 1+outy0+traceydispm[w], color=rgb(0,255,0), /device, linestyle=0

   xyouts, outx0+5, outy0+ysize-10, removepath(repstr(skysubfile,'.fits',''))+ ' ('+target+')', charsize=1, /device
   xyouts, outx0+900, outy0+ysize-10, clip(gra)+' w='+clip(fpr(cwav,5.1)), charsize=1, /device, color=rgb(220,220,220)

   pa = sxpar(hs,'SLITPA')
   parad = pa*!pi/180.
   r = 14.
   plots, outx0+1024+25-2, outy0+[0,ysize-1], /device, color=rgb(100,100,255)
   plots, outx0+1024+25+2, outy0+[0,ysize-1], /device, color=rgb(100,100,255)
   plots, outx0+1024+25 + [0,r*sin(parad)] ,     outy0+ysize/2+ [0, r*cos(parad)], /device, thick=2          ; north bar
   plots, outx0+1024+25 + [0,r*sin(parad-!pi/2.)] ,  outy0+ysize/2+ [0, r*cos(parad-!pi/2.)], /device    ; east bar
   xyouts, outx0+1024+25+1.28*r*sin(parad) - 3., outy0+ysize/2+1.28*r*cos(parad) - 3., /device, 'N', charsize=0.7
   xyouts, outx0+1024+7, outy0+2, /device, 'PA='+clip(round(float(pa))), charsize=0.75
end


pro lrisplottraces, camera=camera

   ; make this postscript, being blind to this is problematic

   common lrisauto, autoastrocommand, swarpcommand, datadir, imworkingdir, spworkingdir2d, spworkingdir1d, imfinaldir, spfinaldir, lrisversion, wildchar, overwrite, fileseq, filterstr, gratingstr, targetstr, defaulttraceyred, defaulttraceyblue, displayoff, debug

   camchar = strmid(camera,0,1)
   chipchar = 'r'
   specfiles = file_search(spworkingdir1d+'els'+camchar+'*.spec')
   imfiles = spworkingdir1d+repstr(strmid(removepath(specfiles),1),'.spec','.fits')  
   ;specfiles = choosefiles(camchar+'*'+'.spec',spworkingdir+'eszfp',spworkingdir+'esfp')
     ; formerly '*' was wildchar+chipchar; this should be safe and tolerates user-specific summed extractions
   ;imfiles = repstr(repstr(specfiles,'.spec','.fits'),'esz','sz') ; im means 2D spectrum
   ; could invert the order to show unannotated images for failed extractions?
   
   imfileexists = bytarr(n_elements(specfiles))
   for i = 0, n_elements(specfiles)-1 do begin
      if file_test(imfiles[i]) then imfileexists[i] = 1 else begin
         for j = 0, 9 do $
            imfiles[i] = repstr(imfiles[i],'-'+clip(j)+'.fits','.fits') 
         if file_test(imfiles[i]) then imfileexists[i] = 1 else  $
            print, 'No FITS file matching '+specfiles[i]
      endelse
   endfor
   nfiles = total(imfileexists)
   if nfiles eq 0 then return
   specfiles = specfiles[where(imfileexists)]
   imfiles = imfiles[where(imfileexists)]

   ;imfiles = 
   
   s = readspec(specfiles[0])
   ; medy = round(median(s.ypix, /even))  ; if uncommented, use a common frame for everything


   yboxsize = 50
   yhead = 5
   yfoot = 5
   outx0 = 5
   xcompass = 50
   yborder = 1
   xwin = 1024+outx0+xcompass
   maxperpage = (600-yhead-yfoot)/(yboxsize+2*yborder)
   xbin = 4
   ybin = 1
   
   p = 0
   for i = 0, nfiles-1 do begin
      pi = (i mod maxperpage)
      if pi eq 0 then begin
         ywin = (nfiles-(maxperpage*p) < maxperpage)*(yboxsize+2*yborder)+yhead+yfoot
         window, 2, xsize=xwin, ysize=ywin    ;, /pixmap
         p += 1
      endif

      outy0 = ywin - yhead - (pi+1)*(yboxsize+2*yborder) + yborder
      lrisplottrace, imfiles[i], specfiles[i], ycenter=medy, outx0=outx0, outy0=outy0, ysize=yboxsize, xbin=xbin, ybin=ybin
      ;;;plots, 5*[i, i], [outy0, outy0+(yboxsize+yborder*2)], /device
      ;print, imfiles[i], specfiles[i], outy0, outy0+yboxsize+2*yborder

      if pi eq maxperpage-1 or i eq nfiles-1 then write_png, spworkingdir1d+'lris'+strmid(camchar,0,1)+'trace'+clip(p)+'.png', tvrd(/true)
   endfor

   tracepngblue = file_search(spworkingdir1d+'lrisbtrace*.png')
   tracepngred = file_search(spworkingdir1d+'lrisrtrace*.png')
   openw, 7, spworkingdir1d+'lristrace.html'
     printf, 7, '<html>'
     printf, 7, '<head>'
     printf, 7, '<title>LPIPE trace check page</title>'
     printf, 7, '</head>'
     printf, 7, '<body>'
     printf, 7, '<table>'
     printf, 7, '<tr><td valign="top">'
     for g = 0, n_elements(tracepngblue)-1 do  printf, 7, '<img src="'+removepath(tracepngblue[g])+'"/>'
     printf, 7, '</td><td valign="top">'
     for g = 0, n_elements(tracepngred)-1 do   printf, 7, '<img src="'+removepath(tracepngred[g])+'"/>'
     printf, 7, '</td>'
     printf, 7, '</body>'
     printf, 7, '</html>'
   close, 7


end


;function srebin, image, xbin, ybin, average=average ;, sum=sum
;  ; I didn't trust the default IDL packages to rebin properly, but they do seem to work.
;
;  nx = (size(image)) [1]
;  ny = (size(image)) [2]
;  
;  nxnew = fix(nx / xbin)
;  nynew = fix(ny / ybin)
;
;  binimage = fltarr(nxnew, nynew)
;
;  for xn = 0, nxnew-1 do begin
;    for yn = 0, nynew-1 do begin
;      binimage[xn, yn] = total(image[xbin*xn:xbin*xn+xbin-1, ybin*yn:ybin*yn+ybin-1])
;    endfor
;  endfor
;
;  if keyword_set(average) then binimage /= (xbin*ybin)
; 
;  return, binimage
;
;end

