pro lrisrename, filestr, check=check

  if n_elements(filestr) eq 0 then begin
     print, 'Specify files to rename (or file wildcard string).'
     print, "   For KOA files, try: 'L?.*.fits'"
     return
  endif
  ct = 1
  if n_elements(filestr) eq 1 then files = file_search(filestr, count=ct) else files = filestr
  if ct eq 0 or file_test(filestr[0]) eq 0 then begin
     print, 'File(s) not found: ', filestr
  endif

  outfiles = ['']
  alphabet = ['','b','c','d','e','f','g','h','i','j']
 
  prevfilenum = -1
  for f = 0, n_elements(files)-1 do begin
     if files[f] eq '' then begin
        print, 'No files matching ', filesstr
     endif
     h = headfits(files[f])

     filenum = sxpar(h,'FRAMENO', count=ct)
     if ct eq 0 then begin
        print, 'WARNING: File header is missing frame number! ', files[f]
        print, '  Not renaming.'
        continue
     endif
     filenumstr = string(filenum,'(I04)')
     instrume = clip(sxpar(h,'INSTRUME'))
     inststr = ''
     if instrume eq 'LRIS' then inststr = 'r'
     if instrume eq 'LRISBLUE' then inststr = 'b'
     if inststr eq '' then begin
       print, 'Cannot recognize instrument.'
       continue
     endif
     datestr = ''
     utdate = sxpar(h,'DATE', /silent, count=ct)
     if ct gt 0 then begin
        utdate = (strsplit(utdate,'T',/extract)) [0]
        if strpos(utdate,'-') gt 0 then datestr = repstr(strmid(utdate,2),'-','') else $
        if strpos(utdate,'/') gt 0 then datestr = repstr(strmid(utdate,2),'/','')
     endif
     if datestr eq '' then print, 'WARNING: Cannot recognize UT date from file header'

     outname = inststr + datestr + '_' + filenumstr + '.fits'

     a = 0
     while total(outname eq outfiles) gt 0 do begin 
        ; we already wrote a file with this name
        a += 1        
        outname = inststr + datestr + '_' + filenumstr + alphabet[a]+'.fits'     
     endwhile
     
     if file_test(outname) then begin
        print, 'WARNING: File already exists!: ', outname
        print, '   Not overwriting.'
        continue
     endif
     outfiles = [outfiles, outname]
  
     print, files[f], ' -> ', outname
     if keyword_set(check) eq 0 then spawn, 'mv '+files[f]+' '+outname 

  endfor


end

