function readlrisheaders, files, crval=crval, badarclist=badarclist, silent=silent
 
  if n_elements(files) eq 0 then begin
     print, 'No files'
     return, -1
  endif

  lris = replicate({filename:'', framenum:-1, date:'', ut:'', jd:0.D, mjd:0.D, mjdstop:0.D, exptime:!values.f_nan, elaptime:!values.f_nan, exp:!values.f_nan, sunelev:!values.f_nan, moonelev:!values.f_nan, moondist:!values.f_nan,  object:'', target:'',  rastr:'', decstr:'', radeg:0.D, decdeg:0.D, airmass:!values.f_nan, azimuth:!values.f_nan, elevation:!values.f_nan, domeazimuth:!values.f_nan, domerelaz:!values.f_nan, slitpa:!values.f_nan, drive:!values.f_nan, instrument:'', camera:'', trapdoor:'', slit:'', dichroic:'', grname:'', filter:'', lamps:'', hg:0b, neon:0b, ar:0b, cd:0b, zn:0b, hal:0b, kr:0b, xe:0b, fe:0b, deu:0b, wavelength:!values.f_nan, cwave:!values.f_nan, skycounts:!values.f_nan, maxcounts:!values.f_nan, ctrate:!values.f_nan, maxsource:!values.f_nan, window:'', pane:'', win:'', chip:'', binning:'', bin:'',  nx:0, ny:0, x0:0, y0:0, xbin:0, ybin:0, pxscale:!values.f_nan, dispersion:!values.f_nan, satval:!values.f_nan, dichf:'', rfiltf:'', bfiltf:'', gratf:'', granglef:''}, n_elements(files))  ; much more to come
 
  ; get important header information for a block of files.

   for f = 0, n_elements(files)-1 do begin
     if files[f] eq '' then continue
     if file_test(files[f]) eq 0 then begin
        if keyword_set(silent) eq 0 then print, 'Cannot find file ', files[f]
        continue 
     endif

     lris[f].filename = files[f]
     if strpos(files[f],'.fits') ge 0 then h = headfits(files[f], /silent)
     if strpos(files[f],'.spec') ge 0 then h = headspec(files[f])
     if strpos(files[f],'.fits') lt 0 and strpos(files[f],'.spec') lt 0 then h = headread(files[f])
     lris[f].framenum = sxpar(h,'FRAMENO')

       ; Date/time
     lris[f].date = clip(sxpar(h,'DATE', /silent))   ; date string, e.g. "2013-01-13T08:48:37"
     lris[f].ut   = clip(sxpar(h,'UT', /silent, count=ct))
        if ct eq 0 then lris[f].ut   = clip(sxpar(h,'UTC', /silent, count=ct))
     lris[f].jd   = double(sxpar(h,'JD')) ; strings are not read properly by SXPAR, unfortunately, so the precision is lost.
     lris[f].mjd   = double(sxpar(h,'MJD-OBS')) 
     ; lris[f].utstop   = clip(sxpar(h,'UTC-END', /silent, count=ct))

       ; Exposure
     lris[f].exptime  = float(sxpar(h,'EXPTIME'))   ; effective time on source (what you would divide by to get the electron rate)
     lris[f].elaptime = (float(sxpar(h,'ELAPTIME')) > float(sxpar(h,'TTIME'))) > float(sxpar(h,'TOTALEXP')) ; including pauses and exposure gaps
     lris[f].exp = lris[f].exptime                                                                          ; currently imperfect
     lris[f].mjdstop  = lris[f].mjd + lris[f].elaptime/(24.*3600.) ; really need to a better job of this for stacks

       ; Astronomical ephemeris
     lris[f].sunelev  = sxpar(h,'SUNELEV')
     lris[f].moonelev = sxpar(h,'MOONELEV')
     lris[f].moondist = sxpar(h,'MOONDIST')
 
       ; Target and sky position
     lris[f].object = clip(sxpar(h,'OBJECT'))
     lris[f].target = repstr(clip(sxpar(h,'TARGNAME')),' ', '_')  ; spaces cause barfing in filenames
     lris[f].rastr  = clip(sxpar(h,'RA'))
     lris[f].decstr = clip(sxpar(h,'DEC'))
     lris[f].radeg  =  ten(sxpar(h,'RA'))*15.
     lris[f].decdeg =  ten(sxpar(h,'DEC'))
     if keyword_set(crval) then lris[f].radeg  = sxpar(h,'CRVAL1')
     if keyword_set(crval) then lris[f].decdeg = sxpar(h,'CRVAL2')

       ; Telescope position
     lris[f].airmass = sxpar(h,'AIRMASS')
     lris[f].azimuth = sxpar(h,'AZ')
     lris[f].elevation = sxpar(h,'EL')
     lris[f].domeazimuth = sxpar(h,'DOMEPOSN')
     lris[f].domerelaz = lris[f].domeazimuth - lris[f].azimuth
        while lris[f].domerelaz lt -180.0 do lris[f].domerelaz += 360.
        while lris[f].domerelaz ge  180.0 do lris[f].domerelaz -= 360.
        if lris[f].domeazimuth eq 0.0 then lris[f].domerelaz = 0.0  ; Sometimes DOMEPOSN is missing and causes data to be excluded.
     lris[f].slitpa = sxpar(h,'SLITPA')                             ; need a more robust solution.
     lris[f].drive = sxpar(h,'ROTPPOSN') ; is this really the drive angls?

       ; Instrument state
     lris[f].instrument = clip(sxpar(h, 'INSTRUME'))             ; instrument name, "LRIS" or "LRISBLUE"
     if lris[f].instrument eq 'LRISBLUE' then lris[f].camera = 'blue'
     if lris[f].instrument eq 'LRIS' then     lris[f].camera = 'red'
     lris[f].trapdoor   = clip(sxpar(h,'TRAPDOOR'))              ; trapdoor state, "open" or "closed"
     lris[f].slit       = clip(sxpar(h, 'SLITNAME'))             ; slit name, e.g. "long_1.0" or "direct"
     lris[f].dichroic   = clip(string(sxpar(h,'DICHNAME')))
     grkey = ''
     if lris[f].instrument eq 'LRISBLUE' then grkey = 'GRISNAME'
     if lris[f].instrument eq 'LRIS'     then grkey = 'GRANAME'
     if grkey ne '' then lris[f].grname = clip(sxpar(h,grkey)) $
                    else lris[f].grname = clip(sxpar(h,'GRISNAME'))+'+'+clip(sxpar(h,'GRANAME'))
     fkey = ''
     if lris[f].instrument eq 'LRISBLUE' then fkey = 'BLUFILT'
     if lris[f].instrument eq 'LRIS'     then fkey = 'REDFILT'
     if lris[f].instrument eq 'LRISBLUE' and clip(lris[f].grname,2) eq 'NB' then fkey = 'GRISNAME'
     if fkey ne '' then lris[f].filter = clip(sxpar(h,fkey))
     lampstr = clip(sxpar(h,'LAMPS'))
     if strpos(lampstr,'1') ge 0 then begin
        if n_elements(badarclist) gt 0 then fixlampstr, lampstr, badarclist
        lamps = fix(strsplit(lampstr,',',/extract))
        lris[f].hg =   lamps[0]
        lris[f].neon = lamps[1]
        lris[f].ar =   lamps[2]
        lris[f].cd =   lamps[3]
        lris[f].zn =   lamps[4]
        lris[f].hal =  lamps[5]
        ; additional (new) lamps?  not sure about the order (kr, xe, fear, deu)
     endif
     lris[f].lamps = lampstr

     lris[f].wavelength = sxpar(h,'WAVELEN') 
     lris[f].cwave = sxpar(h,'CWAVE') 
     if lris[f].cwave le 0. then begin ; backwards compatibility
      if lris[f].camera eq 'r' then begin
        lris[f].cwave = sxpar(h,'WAVELEN')
      endif else begin
        if lris[f].grname eq '300/5000' then lris[f].cwave = 4495.
        if lris[f].grname eq '400/3400' then lris[f].cwave = 3600. 
        if lris[f].grname eq '600/4000' then lris[f].cwave = 4305.
        if lris[f].grname eq '1200/3400' then lris[f].cwave = 3400.
      endelse   
     endif

       ; Background-related keywords
     lris[f].skycounts = sxpar(h,'SKYMED', /silent, count=ct)
       if ct eq 0 then lris[f].skycounts = sxpar(h,'COUNTS')
     lris[f].ctrate  = sxpar(h,'SKYRATE', /silent, count=ct)
       if ct eq 0 then lris[f].ctrate  = sxpar(h,'CTRATE')
     lris[f].maxcounts =  sxpar(h,'MAXCOUNT')
     lris[f].maxsource = sxpar(h,'SOURCMAX', /silent, count=ct)
        if ct eq 0 then lris[f].maxsource =  sxpar(h,'MAXCOUNT')-lris[f].skycounts

       ; CCD keywords
     lris[f].window = clip(sxpar(h,'WINDOW'))
     lris[f].pane   = clip(sxpar(h,'PANE', count=ct))
     if ct gt 0 then lris[f].win = clip(lris[f].pane) else $
                     lris[f].win = strmid(clip(lris[f].window),2)
     lris[f].chip = clip(sxpar(h, 'CHIP'))
     lris[f].binning = repstr(clip(sxpar(h,'BINNING')),' ','')
     lris[f].bin = repstr(lris[f].binning,',','x')
     lris[f].nx = sxpar(h,'NAXIS1')
     lris[f].ny = sxpar(h,'NAXIS2')
     lris[f].x0 = -fix(sxpar(h,'LTV1')) ; theoretically degenerate with pane...
     lris[f].y0 = -fix(sxpar(h,'LTV2'))
     lris[f].xbin = sxpar(h,'XBIN')
     lris[f].ybin = sxpar(h,'YBIN')

      ; First-of-config file references
     lris[f].dichf = sxpar(h,'DICHSEQF') 
     lris[f].rfiltf = sxpar(h,'RFLTSEQF')
     lris[f].bfiltf = sxpar(h,'BFLTSEQF')
     lris[f].gratf = sxpar(h,'GRATSEQF') 
     lris[f].granglef = sxpar(h,'GRANSEQF')

      ; Other
     lris[f].pxscale = sxpar(h,'PXSCALE')
     lris[f].dispersion = sxpar(h,'DISPERSN')
     lris[f].satval =  sxpar(h,'SATURATE')


   endfor

   return, lris

end

